Package: factorselect
Title: Eigenvalue-Based Estimation of the Number of Factors in
        Approximate Factor Models
Version: 0.1.3
Authors@R: person("Jason", "Parker", email = "jparker588@gmail.com",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0001-9227-6976"))
Description: Eigenvalue-based estimation of the number of factors in
    approximate factor models. Designed to work when either N or T is
    large, without requiring both dimensions to grow simultaneously.
    Implements the eigenvalue ratio estimator of Ahn and Horenstein
    (2013) <doi:10.3982/ECTA8968>, the information criteria of Bai and
    Ng (2002) <doi:10.1111/1468-0262.00273>, the tuned penalty of
    Alessi, Barigozzi and Capasso (2010) <doi:10.1016/j.spl.2010.08.005>,
    the auto-covariance ratio estimator of Lam and Yao (2012)
    <doi:10.1214/12-AOS970>, and the edge distribution estimators of
    Onatski (2009) <doi:10.3982/ECTA6964> and Onatski (2010)
    <doi:10.1162/REST_a_00043>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: RSpectra, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/penny4nonsense/factorselect
BugReports: https://github.com/penny4nonsense/factorselect/issues
NeedsCompilation: no
Packaged: 2026-04-24 19:22:51 UTC; e200601
Author: Jason Parker [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9227-6976>)
Maintainer: Jason Parker <jparker588@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:20:14 UTC
Built: R 4.6.0; ; 2026-04-28 20:29:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,factor_select)
S3method(print,factor_select)
S3method(summary,factor_select)
export(select_factors)
export(simulate_factor_model)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(stats,rnorm)
importFrom(stats,sd)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# factorselect

An R package for eigenvalue-based estimation of the number of factors in
approximate factor models. Designed to work when either N or T is large,
without requiring both dimensions to grow simultaneously.

## Installation

``` r
# Development version from GitHub
devtools::install_github("penny4nonsense/factorselect")
```

## Basic Usage

``` r
library(factorselect)

# Simulate a factor model with 3 true factors
X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5, seed = 42)

# Estimate number of factors using Ahn & Horenstein (2013)
result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
print(result)
#> Factor Number Selection
#> =======================
#> Call: select_factors(X = X, method = "ahn_horenstein", kmax = 8)
#> 
#> kmax: 8 
#> 
#> Estimated number of factors:
#>   ahn_horenstein       3
```

## All Methods

``` r
# Run all estimators at once
result <- select_factors(X,
                         method = c("ahn_horenstein", "bai_ng", "abc",
                                    "lam_yao", "onatski_2009", "onatski_2010"),
                         kmax   = 8)
print(result)
#> Factor Number Selection
#> =======================
#> Call: select_factors(X = X, method = c("ahn_horenstein", "bai_ng", 
#>     "abc", "lam_yao", "onatski_2009", "onatski_2010"), kmax = 8)
#> 
#> kmax: 8 
#> 
#> Estimated number of factors:
#>   ahn_horenstein       3
#>   bai_ng               3
#>   abc                  3
#>   lam_yao              6
#>   onatski_2009         3
#>   onatski_2010         3
```

## Scree Plot

``` r
# Visualize the eigenvalue spectrum
result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
plot(result)
```

<img src="man/figures/README-plot-1.png" alt="" width="100%" />

## Methods Implemented

| Method                        | Reference                          |
|-------------------------------|------------------------------------|
| Eigenvalue Ratio (ER, GR)     | Ahn & Horenstein (2013)            |
| Information Criteria (PC, IC) | Bai & Ng (2002)                    |
| Tuned Information Criteria    | Alessi, Barigozzi & Capasso (2010) |
| Auto-covariance Ratio         | Lam & Yao (2012)                   |
| Edge Distribution Test        | Onatski (2009)                     |
| Edge Distribution Estimator   | Onatski (2010)                     |

## References

Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. *Econometrica*, 81(3), 1203-1227.

Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. *Econometrica*, 70(1), 191-221.

Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
for Determining the Number of Factors in Approximate Factor Models.
*Statistics and Probability Letters*, 80, 1806-1813.

Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional Time
Series: Inference for the Number of Factors. *The Annals of Statistics*,
40(2), 694-726.

Onatski, A. (2009). Testing Hypotheses About the Number of Factors in
Large Factor Models. *Econometrica*, 77(5), 1447-1479.

Onatski, A. (2010). Determining the Number of Factors From Empirical
Distribution of Eigenvalues. *The Review of Economics and Statistics*,
92(4), 1004-1016.

#' Alessi, Barigozzi and Capasso (2010) Tuned Information Criteria
#'
#' Estimates the number of factors using the tuning-stability procedure of
#' Alessi, Barigozzi and Capasso (2010) applied to the three IC penalty
#' functions of Bai and Ng (2002). For each penalty function, a grid of
#' tuning constants is used and the most stable estimate across the grid
#' is selected as the final estimate.
#'
#' @param eigenvalues Numeric vector of eigenvalues in descending order of
#'   length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.
#' @param V0 Numeric scalar. Total mean squared value of the panel,
#'   \code{sum(X^2) / (N * TT)}, computed from unstandardized demeaned data.
#' @param kmax Integer. Maximum number of factors to consider.
#' @param N Integer. Number of cross-sectional units.
#' @param TT Integer. Number of time periods.
#' @param c_grid Numeric vector. Grid of tuning constants over which to
#'   evaluate stability. Defaults to \code{seq(0, 1, by = 0.01)}.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k_abc1}{Integer. Selected number of factors using ABC with IC1 penalty.}
#'     \item{k_abc2}{Integer. Selected number of factors using ABC with IC2 penalty.}
#'     \item{k_abc3}{Integer. Selected number of factors using ABC with IC3 penalty.}
#'     \item{k_grid_abc1}{Integer vector of length \code{length(c_grid)}.
#'       Selected k for each value of c using IC1 penalty.}
#'     \item{k_grid_abc2}{Integer vector of length \code{length(c_grid)}.
#'       Selected k for each value of c using IC2 penalty.}
#'     \item{k_grid_abc3}{Integer vector of length \code{length(c_grid)}.
#'       Selected k for each value of c using IC3 penalty.}
#'     \item{c_grid}{Numeric vector. The tuning constant grid used.}
#'   }
#'
#' @details
#'   The ABC estimator applies the tuning-stability procedure of Hallin and
#'   Liska (2007) to the IC criteria of Bai and Ng (2002). For each tuning
#'   constant \eqn{c} in the grid, a modified criterion is minimized:
#'   \deqn{IC_j(k, c) = \ln(V(k)) + k \cdot c \cdot g_j(N, T)}
#'   where \eqn{g_j} is the penalty function from \eqn{IC_{pj}} of Bai and
#'   Ng (2002), for j = 1, 2, 3. The final estimate is the modal value of
#'   \eqn{\hat{k}(c)} across the grid — the value of k that is selected
#'   most frequently as c varies.
#'
#'   As with \code{.bai_ng()}, this estimator requires unstandardized
#'   data. The argument \code{V0} should be computed from demeaned but
#'   unstandardized data.
#'
#'   The ABC estimator generally outperforms the raw Bai & Ng IC criteria
#'   in finite samples, particularly when errors are cross-sectionally
#'   correlated.
#'
#' @references
#'   Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
#'   for Determining the Number of Factors in Approximate Factor Models.
#'   \emph{Statistics and Probability Letters}, 80, 1806-1813.
#'
#'   Bai, J. and Ng, S. (2002). Determining the Number of Factors in
#'   Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
#'
#'   Hallin, M. and Liska, R. (2007). Determining the Number of Factors in
#'   the Generalized Dynamic Factor Model. \emph{Journal of the American
#'   Statistical Association}, 102, 603-617.
#'
#' @seealso \code{.bai_ng()}, \code{.extract_eigenvalues()},
#'   \code{\link{select_factors}}
.abc <- function(eigenvalues, V0, kmax, N, TT,
                 c_grid = seq(0, 1, by = 0.01)) {

  m  <- min(N, TT)
  NT <- N * TT

  # V(k) for k = 0, 1, ..., kmax — same as Bai & Ng
  cumulative <- cumsum(eigenvalues[1:kmax])
  V          <- V0 - c(0, cumulative) / N

  # Base penalty functions from Bai & Ng IC criteria
  g1 <- ((N + TT) / NT) * log(NT / (N + TT))
  g2 <- ((N + TT) / NT) * log(m)
  g3 <- log(m) / m

  ks <- 0:kmax

  # For each c in grid, compute k_hat for each penalty function
  k_grid_abc1 <- integer(length(c_grid))
  k_grid_abc2 <- integer(length(c_grid))
  k_grid_abc3 <- integer(length(c_grid))

  for (i in seq_along(c_grid)) {
    c <- c_grid[i]
    k_grid_abc1[i] <- which.min(log(V) + ks * c * g1) - 1L
    k_grid_abc2[i] <- which.min(log(V) + ks * c * g2) - 1L
    k_grid_abc3[i] <- which.min(log(V) + ks * c * g3) - 1L
  }

  # Modal value across grid
  .mode <- function(x) as.integer(names(which.max(table(x))))

  list(
    k_abc1      = .mode(k_grid_abc1),
    k_abc2      = .mode(k_grid_abc2),
    k_abc3      = .mode(k_grid_abc3),
    k_grid_abc1 = k_grid_abc1,
    k_grid_abc2 = k_grid_abc2,
    k_grid_abc3 = k_grid_abc3,
    c_grid      = c_grid
  )
}

# tests/testthat/test-abc.R

test_that(".abc returns a list with correct elements", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_true(is.list(result))
  expect_named(result, c("k_abc1", "k_abc2", "k_abc3",
                         "k_grid_abc1", "k_grid_abc2", "k_grid_abc3",
                         "c_grid"))
})

test_that("k estimates are non-negative integers within range", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  k_vals <- c(result$k_abc1, result$k_abc2, result$k_abc3)
  expect_true(all(k_vals >= 0))
  expect_true(all(k_vals <= 8))
})

test_that("k_grid vectors have correct length", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  c_grid <- seq(0, 1, by = 0.01)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200,
                 c_grid = c_grid)
  expect_length(result$k_grid_abc1, length(c_grid))
  expect_length(result$k_grid_abc2, length(c_grid))
  expect_length(result$k_grid_abc3, length(c_grid))
})

test_that("c_grid is returned correctly", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  c_grid <- seq(0, 1, by = 0.01)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200,
                 c_grid = c_grid)
  expect_equal(result$c_grid, c_grid)
})

test_that("k_grid values are within valid range", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_true(all(result$k_grid_abc1 >= 0 & result$k_grid_abc1 <= 8))
  expect_true(all(result$k_grid_abc2 >= 0 & result$k_grid_abc2 <= 8))
  expect_true(all(result$k_grid_abc3 >= 0 & result$k_grid_abc3 <= 8))
})

test_that("k_abc1 is the modal value of k_grid_abc1", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_abc1,
               as.integer(names(which.max(table(result$k_grid_abc1)))))
})

test_that("k_abc2 is the modal value of k_grid_abc2", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_abc2,
               as.integer(names(which.max(table(result$k_grid_abc2)))))
})

test_that("k_abc3 is the modal value of k_grid_abc3", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_abc3,
               as.integer(names(which.max(table(result$k_grid_abc3)))))
})

test_that("c = 0 always selects k = kmax", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200,
                 c_grid = 0)
  # With c=0 penalty vanishes so log(V) is minimized at kmax
  expect_equal(result$k_grid_abc1, 8L)
  expect_equal(result$k_grid_abc2, 8L)
  expect_equal(result$k_grid_abc3, 8L)
})

test_that("custom c_grid is respected", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  c_grid <- seq(0, 1, by = 0.1)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200,
                 c_grid = c_grid)
  expect_length(result$k_grid_abc1, length(c_grid))
  expect_equal(result$c_grid, c_grid)
})

test_that("ABC1 recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_equal(result$k_abc1, k_true)
})

test_that("ABC2 recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_equal(result$k_abc2, k_true)
})

test_that("ABC3 recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 500; TT <- 1000; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  # ABC3 uses the weakest penalty (log(m)/m) and requires large samples
  # Accept k_true or adjacent values as reasonable
  expect_true(result$k_abc3 %in% c(k_true - 1L, k_true, k_true + 1L))
})

test_that("works with N > TT", {
  set.seed(42)
  N <- 200; TT <- 50; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_true(is.list(result))
})

test_that("works with TT > N", {
  set.seed(42)
  N <- 50; TT <- 200; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .abc(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_true(is.list(result))
})

#' Onatski (2010) Edge Distribution Estimator
#'
#' Estimates the number of factors using the Edge Distribution (ED)
#' estimator of Onatski (2010). The estimator exploits the fact that
#' idiosyncratic eigenvalues of the sample covariance matrix cluster
#' around a single point, while systematic eigenvalues diverge to
#' infinity. The threshold separating the two groups is estimated
#' iteratively using the square root shape of the edge of the
#' eigenvalue distribution.
#'
#' @param eigenvalues Numeric vector of eigenvalues in descending order,
#'   typically obtained from \code{.extract_eigenvalues()}. Must
#'   contain at least \code{kmax + 5} elements to allow the OLS
#'   regression in the calibration step.
#' @param kmax Integer. Maximum number of factors to consider.
#' @param n_iter Integer. Maximum number of iterations for the
#'   calibration procedure. Defaults to \code{4} as recommended
#'   by Onatski (2010).
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k}{Integer. Estimated number of factors.}
#'     \item{delta}{Numeric. The estimated threshold \eqn{\delta = 2|\hat{\beta}|}.}
#'     \item{beta}{Numeric. The estimated slope coefficient \eqn{\hat{\beta}}
#'       from the OLS regression in the final iteration.}
#'     \item{differences}{Numeric vector of length kmax. Successive
#'       eigenvalue differences \eqn{\lambda_i - \lambda_{i+1}}.}
#'     \item{n_iter}{Integer. Number of iterations performed.}
#'   }
#'
#' @details
#'   The ED estimator of Onatski (2010) is based on the theoretical result
#'   that idiosyncratic eigenvalues cluster around the upper edge
#'   \eqn{u(\mathcal{F}^{c,A,B})} of the limiting spectral distribution,
#'   while systematic eigenvalues diverge. Near the edge, the density of
#'   the limiting spectral distribution behaves like a square root function,
#'   implying that eigenvalue differences \eqn{\lambda_i - \lambda_{i+1}}
#'   for idiosyncratic eigenvalues behave approximately as
#'   \eqn{(an)^{-2/3}}.
#'
#'   The calibration procedure estimates \eqn{\hat{\beta} = (an)^{-2/3}}
#'   by regressing five consecutive eigenvalues \eqn{\lambda_j, \ldots,
#'   \lambda_{j+4}} on a constant and \eqn{(j-1)^{2/3}, \ldots,
#'   (j+3)^{2/3}}, where \eqn{j} is initialized at \eqn{r_{max} + 1}
#'   and updated iteratively.
#'
#'   The estimator requires \code{eigenvalues} to contain at least
#'   \code{kmax + 5} elements so that the OLS window \eqn{j, \ldots, j+4}
#'   is always available.
#'
#' @references
#'   Onatski, A. (2010). Determining the Number of Factors From Empirical
#'   Distribution of Eigenvalues. \emph{The Review of Economics and
#'   Statistics}, 92(4), 1004-1016.
#'
#' @seealso \code{.extract_eigenvalues()}, \code{\link{select_factors}}
.onatski_2010 <- function(eigenvalues, kmax, n_iter = 4L) {

  # Need at least kmax + 5 eigenvalues for the OLS window
  if (length(eigenvalues) < kmax + 5) {
    stop("eigenvalues must contain at least kmax + 5 = ", kmax + 5,
         " elements. Re-run .extract_eigenvalues() with kmax = ",
         kmax + 4, " or larger.")
  }

  # Successive eigenvalue differences lambda_i - lambda_{i+1}
  diffs <- diff(eigenvalues)               # negative since descending
  diffs <- -diffs[1:kmax]                  # positive differences

  # Iterative calibration procedure
  j     <- kmax + 1L                       # initialize j = rmax + 1
  k_hat <- 0L
  beta  <- NA_real_
  delta <- NA_real_

  for (iter in seq_len(n_iter)) {

    # OLS regression of lambda_j, ..., lambda_{j+4}
    # on constant and (j-1)^(2/3), ..., (j+3)^(2/3)
    idx  <- j:(j + 4L)
    y    <- eigenvalues[idx]
    x    <- ((idx - 1L)^(2/3))
    x_dm <- x - mean(x)
    y_dm <- y - mean(y)

    # OLS slope
    beta  <- sum(x_dm * y_dm) / sum(x_dm^2)
    delta <- 2 * abs(beta)

    # Estimate r(delta)
    exceeds <- which(diffs >= delta)
    k_hat   <- if (length(exceeds) > 0) max(exceeds) else 0L

    # Update j
    j <- k_hat + 1L

    # Ensure j + 4 doesn't exceed available eigenvalues
    if (j + 4L > length(eigenvalues)) {
      j <- length(eigenvalues) - 4L
    }
  }

  list(
    k           = k_hat,
    delta       = delta,
    beta        = beta,
    differences = diffs,
    n_iter      = n_iter
  )
}

# tests/testthat/test-onatski_2010.R

test_that(".onatski_2010 returns a list with correct elements", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_true(is.list(result))
  expect_named(result, c("k", "delta", "beta", "differences", "n_iter"))
})

test_that("k is a non-negative integer within range", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_length(result$k, 1)
  expect_true(result$k >= 0 && result$k <= 8)
})

test_that("delta is positive", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_true(result$delta > 0)
})

test_that("delta equals 2 * abs(beta)", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_equal(result$delta, 2 * abs(result$beta))
})

test_that("differences vector has length kmax", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_length(result$differences, 8)
})

test_that("differences are positive", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_true(all(result$differences >= 0))
})

test_that("n_iter is returned correctly", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8, n_iter = 4)
  expect_equal(result$n_iter, 4)
})

test_that("insufficient eigenvalues throws informative error", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  expect_error(.onatski_2010(eig$values, kmax = 8), "kmax \\+ 5")
})

test_that("different n_iter values run without error", {
  set.seed(42)
  X   <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X   <- .prepare_matrix(X)
  eig <- .extract_eigenvalues(X, kmax = 13)
  expect_no_error(.onatski_2010(eig$values, kmax = 8, n_iter = 1))
  expect_no_error(.onatski_2010(eig$values, kmax = 8, n_iter = 4))
  expect_no_error(.onatski_2010(eig$values, kmax = 8, n_iter = 10))
})

test_that("more iterations does not break the result", {
  set.seed(42)
  X       <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X       <- .prepare_matrix(X)
  eig     <- .extract_eigenvalues(X, kmax = 13)
  result2 <- .onatski_2010(eig$values, kmax = 8, n_iter = 2)
  result4 <- .onatski_2010(eig$values, kmax = 8, n_iter = 4)
  # Both should be valid estimates
  expect_true(result2$k >= 0 && result2$k <= 8)
  expect_true(result4$k >= 0 && result4$k <= 8)
})

test_that("recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 13)
  result <- .onatski_2010(eig$values, kmax = 8)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — small N large TT", {
  set.seed(42)
  N <- 25; TT <- 200; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 10)
  result <- .onatski_2010(eig$values, kmax = 5)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — large N small TT", {
  set.seed(42)
  N <- 200; TT <- 50; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 10)
  result <- .onatski_2010(eig$values, kmax = 5)
  expect_equal(result$k, k_true)
})

test_that("works with N equal to TT", {
  set.seed(42)
  X   <- simulate_factor_model(N = 100, TT = 100, k = 3, sd = 0.5)
  X   <- .prepare_matrix(X)
  eig <- .extract_eigenvalues(X, kmax = 13)
  expect_no_error(.onatski_2010(eig$values, kmax = 8))
})

test_that("works with N > TT", {
  set.seed(42)
  X   <- simulate_factor_model(N = 200, TT = 50, k = 2, sd = 0.5)
  X   <- .prepare_matrix(X)
  eig <- .extract_eigenvalues(X, kmax = 10)
  expect_no_error(.onatski_2010(eig$values, kmax = 5))
})

#' Simulate Data from an Approximate Factor Model
#'
#' Generates a simulated panel data matrix from a static approximate factor
#' model. Useful for testing and benchmarking factor number estimators.
#'
#' @param N Integer. Number of cross-sectional units.
#' @param TT Integer. Number of time periods. Named \code{TT} to avoid
#'   conflict with the base R function \code{T} (which evaluates to
#'   \code{TRUE}).
#' @param k Integer. True number of factors.
#' @param sd Numeric. Standard deviation of the idiosyncratic error term.
#'   Defaults to \code{1}. Lower values produce stronger signal relative
#'   to noise.
#' @param seed Integer or \code{NULL}. Random seed for reproducibility.
#'   Defaults to \code{NULL} (no seed set).
#'
#' @return A numeric matrix of dimensions \code{TT x N} generated from:
#'   \deqn{X = F \Lambda' + E}
#'   where \eqn{F} is a \code{TT x k} matrix of factors drawn from
#'   \eqn{N(0,1)}, \eqn{\Lambda} is an \code{N x k} matrix of loadings
#'   drawn from \eqn{N(0,1)}, and \eqn{E} is a \code{TT x N} matrix of
#'   idiosyncratic errors drawn from \eqn{N(0, sd^2)}.
#'
#' @details
#'   The data generating process follows the standard approximate factor
#'   model of Chamberlain and Rothschild (1983) as used in the simulation
#'   exercises of Ahn and Horenstein (2013). Factors and loadings are
#'   independent standard normal draws. Errors are i.i.d. normal with
#'   mean zero and standard deviation \code{sd}.
#'
#'   The signal-to-noise ratio is controlled by \code{sd} — smaller values
#'   produce a cleaner factor structure that is easier for estimators to
#'   recover. The default \code{sd = 1} matches the baseline simulation
#'   design of Ahn and Horenstein (2013) with \code{theta = 1}.
#'
#' @references
#'   Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
#'   Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
#'
#'   Chamberlain, G. and Rothschild, M. (1983). Arbitrage, Factor Structure,
#'   and Mean-Variance Analysis on Large Asset Markets.
#'   \emph{Econometrica}, 51(5), 1281-1304.
#'
#' @seealso \code{\link{select_factors}}
#'
#' @importFrom stats rnorm
#'
#' @export
#'
#' @examples
#' # Simulate a factor model with 3 factors
#' X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5, seed = 42)
#' dim(X)
#'
#' # Pass directly to select_factors
#' result <- select_factors(X)
#' result$k
simulate_factor_model <- function(N, TT, k, sd = 1, seed = NULL) {

  # Input validation
  if (!is.numeric(N) || length(N) != 1 || N < 2)
    stop("N must be a single integer >= 2.")
  if (!is.numeric(TT) || length(TT) != 1 || TT < 2)
    stop("TT must be a single integer >= 2.")
  if (!is.numeric(k) || length(k) != 1 || k < 1)
    stop("k must be a single integer >= 1.")
  if (k >= min(N, TT))
    stop("k must be less than min(N, TT).")
  if (!is.numeric(sd) || length(sd) != 1 || sd <= 0)
    stop("sd must be a single positive number.")

  if (!is.null(seed)) set.seed(seed)

  Lambda <- matrix(rnorm(N  * k),  N,  k)
  F_mat  <- matrix(rnorm(TT * k),  TT, k)
  E      <- matrix(rnorm(TT * N, sd = sd), TT, N)

  F_mat %*% t(Lambda) + E
}

# tests/testthat/test-onatski_2009.R

test_that(".onatski_2009 returns a list with correct elements", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_true(is.list(result))
  expect_named(result, c("k", "ratios", "eigenvalues", "critical_value", "alpha"))
})

test_that("k is a non-negative integer within range", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_length(result$k, 1)
  expect_true(result$k >= 0 && result$k <= 8)
})

test_that("ratios vector has length kmax", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_length(result$ratios, 8)
})

test_that("eigenvalues vector has length kmax + 2", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_length(result$eigenvalues, 10)
})

test_that("eigenvalues are in descending order", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_true(all(diff(result$eigenvalues) <= 0))
})

test_that("eigenvalues are non-negative", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_true(all(result$eigenvalues >= 0))
})

test_that("alpha is returned correctly", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_equal(result$alpha, 0.05)
})

test_that("critical_value is positive", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_true(result$critical_value > 0)
})

test_that("invalid alpha throws informative error", {
  set.seed(42)
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X <- .prepare_matrix(X)
  expect_error(.onatski_2009(X, kmax = 8, alpha = 0.25), "alpha must be one of")
})

test_that("larger alpha gives smaller critical value", {
  set.seed(42)
  X       <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X       <- .prepare_matrix(X)
  result1 <- .onatski_2009(X, kmax = 8, alpha = 0.01)
  result2 <- .onatski_2009(X, kmax = 8, alpha = 0.10)
  expect_gt(result1$critical_value, result2$critical_value)
})

test_that("odd T is handled by dropping last observation", {
  set.seed(42)
  X <- simulate_factor_model(N = 100, TT = 201, k = 3, sd = 0.5)
  X <- .prepare_matrix(X)
  expect_no_error(.onatski_2009(X, kmax = 8, alpha = 0.05))
})

test_that("all valid alpha levels run without error", {
  set.seed(42)
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
  X <- .prepare_matrix(X)
  alpha_levels <- c(0.15, 0.10, 0.09, 0.08, 0.07, 0.06,
                    0.05, 0.04, 0.03, 0.02, 0.01)
  for (a in alpha_levels) {
    expect_no_error(.onatski_2009(X, kmax = 8, alpha = a))
  }
})

test_that("recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 8, alpha = 0.05)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — small N large TT", {
  set.seed(42)
  N <- 25; TT <- 200; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 5, alpha = 0.05)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — large N small TT", {
  set.seed(42)
  N <- 200; TT <- 50; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X      <- .prepare_matrix(X)
  result <- .onatski_2009(X, kmax = 5, alpha = 0.05)
  expect_equal(result$k, k_true)
})

test_that("stricter alpha produces equal or fewer factors", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X       <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.5)
  X       <- .prepare_matrix(X)
  result1 <- .onatski_2009(X, kmax = 8, alpha = 0.01)
  result2 <- .onatski_2009(X, kmax = 8, alpha = 0.15)
  expect_true(result1$k <= result2$k)
})

test_that("works with N equal to TT", {
  set.seed(42)
  X <- simulate_factor_model(N = 100, TT = 100, k = 3, sd = 0.5)
  X <- .prepare_matrix(X)
  expect_no_error(.onatski_2009(X, kmax = 8, alpha = 0.05))
})

test_that("works with N > TT", {
  set.seed(42)
  X <- simulate_factor_model(N = 200, TT = 50, k = 2, sd = 0.5)
  X <- .prepare_matrix(X)
  expect_no_error(.onatski_2009(X, kmax = 5, alpha = 0.05))
})

# tests/testthat/test-lam_yao.R

test_that(".lam_yao returns a list with correct elements", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_true(is.list(result))
  expect_named(result, c("k", "ratios", "eigenvalues"))
})

test_that("k is a single positive integer within range", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_length(result$k, 1)
  expect_true(result$k >= 1 && result$k <= 8)
})

test_that("ratios vector has length kmax", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_length(result$ratios, 8)
})

test_that("eigenvalues vector has length kmax + 1", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_length(result$eigenvalues, 9)
})

test_that("eigenvalues of M are in descending order", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_true(all(diff(result$eigenvalues) <= 0))
})

test_that("eigenvalues of M are non-negative", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_true(all(result$eigenvalues >= 0))
})

test_that("ratios are positive", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_true(all(result$ratios > 0))
})

test_that("k is the argmax of ratios", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_equal(result$k, which.max(result$ratios))
})

test_that("h = 1 and h = 3 both run without error", {
  set.seed(42)
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X <- .prepare_matrix(X, standardize = FALSE)
  expect_no_error(.lam_yao(X, kmax = 8, h = 1))
  expect_no_error(.lam_yao(X, kmax = 8, h = 3))
})

test_that("larger h gives different result than h = 1", {
  set.seed(42)
  X       <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1)
  X       <- .prepare_matrix(X, standardize = FALSE)
  result1 <- .lam_yao(X, kmax = 8, h = 1)
  result5 <- .lam_yao(X, kmax = 8, h = 5)
  expect_false(identical(result1$eigenvalues, result5$eigenvalues))
})

test_that("recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 1)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — small N large TT", {
  set.seed(42)
  N <- 25; TT <- 200; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 5, h = 1)
  expect_equal(result$k, k_true)
})

test_that("recovers true number of factors — large N small TT", {
  set.seed(42)
  N <- 200; TT <- 25; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 5, h = 1)
  expect_equal(result$k, k_true)
})

test_that("works with N equal to TT", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 100, k = 3, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  expect_no_error(.lam_yao(X, kmax = 8, h = 1))
})

test_that("works with multiple lags h = 5", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 0.1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  result <- .lam_yao(X, kmax = 8, h = 5)
  expect_true(result$k >= 1 && result$k <= 8)
})

# tests/testthat/test-simulate_factor_model.R

test_that("simulate_factor_model returns a matrix", {
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  expect_true(is.matrix(X))
})

test_that("simulate_factor_model returns correct dimensions", {
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  expect_equal(dim(X), c(200, 100))
})

test_that("simulate_factor_model returns numeric matrix", {
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  expect_true(is.numeric(X))
})

test_that("simulate_factor_model returns no missing values", {
  X <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  expect_false(anyNA(X))
})

test_that("seed produces reproducible results", {
  X1 <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  X2 <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  expect_equal(X1, X2)
})

test_that("different seeds produce different results", {
  X1 <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 42)
  X2 <- simulate_factor_model(N = 100, TT = 200, k = 3, seed = 99)
  expect_false(identical(X1, X2))
})

test_that("NULL seed runs without error", {
  expect_no_error(simulate_factor_model(N = 100, TT = 200, k = 3, seed = NULL))
})

test_that("N < 2 throws error", {
  expect_error(simulate_factor_model(N = 1, TT = 200, k = 1), "N must")
})

test_that("TT < 2 throws error", {
  expect_error(simulate_factor_model(N = 100, TT = 1, k = 1), "TT must")
})

test_that("k < 1 throws error", {
  expect_error(simulate_factor_model(N = 100, TT = 200, k = 0), "k must")
})

test_that("k >= min(N, TT) throws error", {
  expect_error(simulate_factor_model(N = 10, TT = 200, k = 10), "k must be less")
})

test_that("sd <= 0 throws error", {
  expect_error(simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0), "sd must")
  expect_error(simulate_factor_model(N = 100, TT = 200, k = 3, sd = -1), "sd must")
})

test_that("higher sd produces higher residual variance", {
  X_low  <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.1, seed = 42)
  X_high <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 2.0, seed = 42)
  expect_gt(var(as.vector(X_high)), var(as.vector(X_low)))
})

test_that("works with small N large TT", {
  expect_no_error(simulate_factor_model(N = 10, TT = 500, k = 2, seed = 42))
})

test_that("works with large N small TT", {
  expect_no_error(simulate_factor_model(N = 500, TT = 10, k = 2, seed = 42))
})

test_that("works with N equal to TT", {
  X <- simulate_factor_model(N = 100, TT = 100, k = 3, seed = 42)
  expect_equal(dim(X), c(100, 100))
})

test_that("factor structure is recoverable by select_factors", {
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5, seed = 42)
  result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
  expect_equal(result$k[["ahn_horenstein"]], 3L)
})

# tests/testthat/test-bai_ng.R

test_that(".bai_ng returns a list with correct elements", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_true(is.list(result))
  expect_named(result, c("k_pc1", "k_pc2", "k_pc3",
                         "k_ic1", "k_ic2", "k_ic3",
                         "pc1",   "pc2",   "pc3",
                         "ic1",   "ic2",   "ic3"))
})

test_that("all k estimates are non-negative integers", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  k_vals <- c(result$k_pc1, result$k_pc2, result$k_pc3,
              result$k_ic1, result$k_ic2, result$k_ic3)
  expect_true(all(k_vals >= 0))
  expect_true(all(k_vals <= 8))
})

test_that("all k estimates are integers", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  k_vals <- c(result$k_pc1, result$k_pc2, result$k_pc3,
              result$k_ic1, result$k_ic2, result$k_ic3)
  expect_true(all(k_vals == as.integer(k_vals)))
})

test_that("criterion vectors have length kmax + 1", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_length(result$pc1, 9)
  expect_length(result$pc2, 9)
  expect_length(result$pc3, 9)
  expect_length(result$ic1, 9)
  expect_length(result$ic2, 9)
  expect_length(result$ic3, 9)
})

test_that("k_pc1 is the argmin of pc1", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_pc1, which.min(result$pc1) - 1L)
})

test_that("k_pc2 is the argmin of pc2", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_pc2, which.min(result$pc2) - 1L)
})

test_that("k_pc3 is the argmin of pc3", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_pc3, which.min(result$pc3) - 1L)
})

test_that("k_ic1 is the argmin of ic1", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_ic1, which.min(result$ic1) - 1L)
})

test_that("k_ic2 is the argmin of ic2", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_ic2, which.min(result$ic2) - 1L)
})

test_that("k_ic3 is the argmin of ic3", {
  set.seed(42)
  X      <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (100 * 200)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = 100, TT = 200)
  expect_equal(result$k_ic3, which.min(result$ic3) - 1L)
})

test_that("IC criteria recover true factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_equal(result$k_ic1, k_true)
  expect_equal(result$k_ic2, k_true)
  expect_equal(result$k_ic3, k_true)
})

test_that("PC criteria recover true factors — large N and T", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_equal(result$k_pc1, k_true)
  expect_equal(result$k_pc2, k_true)
  expect_equal(result$k_pc3, k_true)
})

test_that("criteria are sensitive to kmax as documented", {
  set.seed(42)
  N <- 100; TT <- 200; k_true <- 3
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig8   <- .extract_eigenvalues(X, kmax = 8)
  eig15  <- .extract_eigenvalues(X, kmax = 15)
  res8   <- .bai_ng(eig8$values,  V0 = V0, kmax = 8,  N = N, TT = TT)
  res15  <- .bai_ng(eig15$values, V0 = V0, kmax = 15, N = N, TT = TT)
  expect_true(is.list(res8))
  expect_true(is.list(res15))
})

test_that("works with N > TT", {
  set.seed(42)
  N <- 200; TT <- 50; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_true(is.list(result))
})

test_that("works with TT > N", {
  set.seed(42)
  N <- 50; TT <- 200; k_true <- 2
  X      <- simulate_factor_model(N = N, TT = TT, k = k_true, sd = 1)
  X      <- .prepare_matrix(X, standardize = FALSE)
  V0     <- sum(X^2) / (N * TT)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .bai_ng(eig$values, V0 = V0, kmax = 8, N = N, TT = TT)
  expect_true(is.list(result))
})

#' Onatski (2009) Test for the Number of Factors
#'
#' Estimates the number of factors using the sequential hypothesis testing
#' procedure of Onatski (2009), applied to the static approximate factor
#' model version described in Section 4 of that paper. The test statistic
#' is based on ratios of differences of adjacent eigenvalues of a
#' complex-valued transformation of the data.
#'
#' @param X Numeric matrix of dimensions T x N, typically preprocessed by
#'   \code{.prepare_matrix()}. Must have an even number of rows.
#' @param kmax Integer. Maximum number of factors to consider. Defines the
#'   upper bound k1 in the sequential testing procedure.
#' @param alpha Numeric. Significance level for the sequential test.
#'   Defaults to \code{0.05}. Must be one of \code{0.01, 0.02, 0.03, 0.04,
#'   0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.15}.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k}{Integer. Estimated number of factors from the sequential
#'       testing procedure.}
#'     \item{ratios}{Numeric vector of length kmax. The ratio statistic
#'       \eqn{(\tilde{\gamma}_i - \tilde{\gamma}_{i+1}) /
#'       (\tilde{\gamma}_{i+1} - \tilde{\gamma}_{i+2})} for each i.}
#'     \item{eigenvalues}{Numeric vector of length kmax + 2. Leading
#'       eigenvalues of the complex covariance matrix in descending order.}
#'     \item{critical_value}{Numeric. Critical value used for the test
#'       at the specified significance level.}
#'     \item{alpha}{Numeric. The significance level used.}
#'   }
#'
#' @details
#'   The static approximate factor model version of the Onatski (2009) test
#'   (Section 4) proceeds as follows:
#'
#'   \enumerate{
#'     \item Split the T x N data matrix into two halves of length T/2.
#'     \item Form complex-valued vectors
#'       \eqn{\tilde{X}_j = X_j + i X_{j + T/2}} for \eqn{j = 1, \ldots, T/2}.
#'     \item Compute eigenvalues \eqn{\tilde{\gamma}_i} of
#'       \eqn{\frac{2}{T} \sum_{j=1}^{T/2} \tilde{X}_j \tilde{X}_j^*}.
#'     \item Sequentially test \eqn{H_0: r = k_0} versus
#'       \eqn{H_1: k_0 < r \leq k_{max}} for \eqn{k_0 = 0, 1, \ldots}
#'       using the statistic
#'       \eqn{\tilde{R} = \max_{k_0 < i \leq k_{max}}
#'       (\tilde{\gamma}_i - \tilde{\gamma}_{i+1}) /
#'       (\tilde{\gamma}_{i+1} - \tilde{\gamma}_{i+2})}.
#'     \item Stop when \eqn{H_0} is not rejected. The estimate is the
#'       current \eqn{k_0}.
#'   }
#'
#'   Critical values are taken from Table I of Onatski (2009) and depend
#'   on the significance level \code{alpha} and the number of factors
#'   tested under the alternative \eqn{k_1 - k_0 = k_{max} - k_0}.
#'
#'   If T is odd, the last observation is dropped to ensure equal-length
#'   halves.
#'
#' @references
#'   Onatski, A. (2009). Testing Hypotheses About the Number of Factors
#'   in Large Factor Models. \emph{Econometrica}, 77(5), 1447-1479.
#'
#' @seealso \code{.ahn_horenstein()}, \code{\link{select_factors}}
.onatski_2009 <- function(X, kmax, alpha = 0.05) {

  # Critical values from Table I of Onatski (2009)
  # Rows: significance levels (15%, 10%, 9%, ..., 1%)
  # Columns: k1 - k0 = 1, 2, 3, 4, 5, 6, 7, 8
  .cv_table <- matrix(c(
    2.75, 3.62, 4.15, 4.54, 4.89, 5.20, 5.45, 5.70,
    3.33, 4.31, 4.91, 5.40, 5.77, 6.13, 6.42, 6.66,
    3.50, 4.49, 5.13, 5.62, 6.03, 6.39, 6.67, 6.92,
    3.69, 4.72, 5.37, 5.91, 6.31, 6.68, 6.95, 7.25,
    3.92, 4.99, 5.66, 6.24, 6.62, 7.00, 7.32, 7.59,
    4.20, 5.31, 6.03, 6.57, 7.00, 7.41, 7.74, 8.04,
    4.52, 5.73, 6.46, 7.01, 7.50, 7.95, 8.29, 8.59,
    5.02, 6.26, 6.97, 7.63, 8.16, 8.61, 9.06, 9.36,
    5.62, 6.91, 7.79, 8.48, 9.06, 9.64, 10.11, 10.44,
    6.55, 8.15, 9.06, 9.93, 10.47, 11.27, 11.75, 12.13,
    8.74, 10.52, 11.67, 12.56, 13.42, 14.26, 14.88, 15.25
  ), nrow = 11, byrow = TRUE)

  .alpha_levels <- c(0.15, 0.10, 0.09, 0.08, 0.07, 0.06,
                     0.05, 0.04, 0.03, 0.02, 0.01)

  # Validate alpha
  if (!alpha %in% .alpha_levels) {
    stop("alpha must be one of: ",
         paste(.alpha_levels, collapse = ", "))
  }

  alpha_row <- which(.alpha_levels == alpha)

  TT <- nrow(X)
  N  <- ncol(X)

  # Drop last observation if T is odd
  if (TT %% 2 != 0) {
    X  <- X[1:(TT - 1), ]
    TT <- TT - 1
  }

  T2 <- TT / 2

  # Form complex matrix X_tilde: T/2 x N
  X1 <- X[1:T2, ]
  X2 <- X[(T2 + 1):TT, ]
  X_tilde <- X1 + 1i * X2   # T/2 x N complex

  # Compute eigenvalues of (2/T) * t(Conj(X_tilde)) %*% X_tilde
  # This is an N x N complex Hermitian matrix
  S <- (2 / TT) * Conj(t(X_tilde)) %*% X_tilde

  # Need kmax + 2 eigenvalues for the ratio statistic
  eig     <- eigen(S, symmetric = FALSE)
  eig_val <- Re(eig$values)   # eigenvalues are real for Hermitian matrix
  eig_val <- sort(eig_val, decreasing = TRUE)
  eigs    <- eig_val[1:(kmax + 2)]

  # Compute ratio statistics for all positions
  # ratio[i] = (gamma[i] - gamma[i+1]) / (gamma[i+1] - gamma[i+2])
  ratios <- (eigs[1:kmax] - eigs[2:(kmax + 1)]) /
    (eigs[2:(kmax + 1)] - eigs[3:(kmax + 2)])

  # Sequential testing procedure (Section 5.3)
  # Test H0: r = k0 vs H1: k0 < r <= kmax for k0 = 0, 1, ...
  k_hat <- 0L

  for (k0 in 0:(kmax - 1)) {

    # Number of factors under alternative: k1 - k0 = kmax - k0
    k1_minus_k0 <- kmax - k0

    # Cap at 8 (maximum column in Table I)
    k1_minus_k0_capped <- min(k1_minus_k0, 8L)

    # Critical value from table
    cv <- .cv_table[alpha_row, k1_minus_k0_capped]

    # Test statistic: max ratio over k0 < i <= kmax
    R_stat <- max(ratios[(k0 + 1):kmax])

    # If we fail to reject H0, stop
    if (R_stat <= cv) {
      k_hat <- k0
      break
    }

    # If we reject all the way to kmax - 1, estimate is kmax
    if (k0 == kmax - 1) {
      k_hat <- kmax
    }
  }

  list(
    k              = k_hat,
    ratios         = ratios,
    eigenvalues    = eigs,
    critical_value = .cv_table[alpha_row, min(kmax, 8L)],
    alpha          = alpha
  )
}

#' Demean and Scale a Matrix for Factor Analysis
#'
#' Removes individual means, time means, or both from a numeric matrix,
#' and optionally scales to unit variance. This is the standard preprocessing
#' step required before eigendecomposition in factor number estimation.
#'
#' @param X Numeric matrix of dimensions T x N (time periods x units).
#' @param demean Character string specifying the demeaning method. One of:
#'   \describe{
#'     \item{"both"}{Remove both individual (column) and time (row) means.
#'       This is the recommended default for macro panels.}
#'     \item{"individual"}{Remove individual (column) means only.}
#'     \item{"time"}{Remove time (row) means only.}
#'     \item{"none"}{No demeaning applied.}
#'   }
#' @param standardize Logical. If \code{TRUE} (default), scale each column
#'   to unit variance after demeaning.
#'
#' @return A demeaned and optionally scaled numeric matrix of the same
#'   dimensions as \code{X}.
#'
#' @details
#' When \code{demean = "both"}, the function iterates individual and time
#' demeaning to convergence. This follows the within-transformation used in
#' panel data models.
#'
#' @references
#' Bai, J. and Ng, S. (2002). Determining the Number of Factors in
#' Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
#'
#' @seealso \code{\link{select_factors}}
#'
#' @importFrom stats sd
.prepare_matrix <- function(X, demean = c("both", "individual", "time", "none"),
                            standardize = TRUE) {

  demean <- match.arg(demean)

  # Input validation
  if (!is.matrix(X))        X <- as.matrix(X)
  if (!is.numeric(X))       stop("X must be numeric.")
  if (anyNA(X))             stop("X contains missing values. A balanced panel is required.")
  if (nrow(X) < 2)          stop("X must have at least 2 rows.")
  if (ncol(X) < 2)          stop("X must have at least 2 columns.")

  # Demeaning
  X <- switch(demean,
              individual = {
                X - matrix(colMeans(X), nrow = nrow(X), ncol = ncol(X), byrow = TRUE)
              },
              time = {
                X - rowMeans(X)
              },
              both = {
                # Two-pass within transformation
                X <- X - matrix(colMeans(X), nrow = nrow(X), ncol = ncol(X), byrow = TRUE)
                X <- X - rowMeans(X)
                X
              },
              none = X
  )

  # Standardize columns to unit variance
  if (standardize) {
    col_sd <- apply(X, 2, sd)
    zero_var <- col_sd < .Machine$double.eps
    if (any(zero_var)) {
      warning(sum(zero_var), " column(s) have zero variance after demeaning and will not be scaled.")
      col_sd[zero_var] <- 1
    }
    X <- X / matrix(col_sd, nrow = nrow(X), ncol = ncol(X), byrow = TRUE)
  }

  X
}

# tests/testthat/test-ahn_horenstein.R

test_that(".ahn_horenstein returns a list with correct elements", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_true(is.list(result))
  expect_named(result, c("k_er", "k_gr", "er", "gr"))
})

test_that("k_er is a single positive integer within range", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_length(result$k_er, 1)
  expect_true(result$k_er >= 1 && result$k_er <= 5)
})

test_that("k_gr is a single positive integer within range", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_length(result$k_gr, 1)
  expect_true(result$k_gr >= 1 && result$k_gr <= 5)
})

test_that("er statistic vector has length kmax", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_length(result$er, 5)
})

test_that("gr statistic vector has length kmax", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_length(result$gr, 5)
})

test_that("er statistic values are positive", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_true(all(result$er > 0))
})

test_that("gr statistic values are positive", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_true(all(result$gr > 0))
})

test_that("k_er is the argmax of er", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_equal(result$k_er, which.max(result$er))
})

test_that("k_gr is the argmax of gr", {
  set.seed(42)
  X      <- matrix(rnorm(200), 20, 10)
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(nrow(X), ncol(X)))
  expect_equal(result$k_gr, which.max(result$gr))
})

test_that("ER recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .ahn_horenstein(eig$values, kmax = 8, n = min(N, T))
  expect_equal(result$k_er, k_true)
})

test_that("GR recovers true number of factors — large N and T", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .ahn_horenstein(eig$values, kmax = 8, n = min(N, T))
  expect_equal(result$k_gr, k_true)
})

test_that("ER recovers true number of factors — small N large T", {
  set.seed(42)
  N <- 25; T <- 200; k_true <- 2
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 8)
  result <- .ahn_horenstein(eig$values, kmax = 8, n = min(N, T))
  expect_equal(result$k_er, k_true)
})

test_that("GR recovers true number of factors — large N small T", {
  set.seed(42)
  N <- 200; T <- 10; k_true <- 2
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  X      <- .prepare_matrix(X)
  eig    <- .extract_eigenvalues(X, kmax = 5)
  result <- .ahn_horenstein(eig$values, kmax = 5, n = min(N, T))
  expect_equal(result$k_gr, k_true)
})

# tests/testthat/test-extract_eigenvalues.R

test_that(".extract_eigenvalues returns a list with correct elements", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_true(is.list(result))
  expect_named(result, c("values", "vectors", "orientation"))
})

test_that("eigenvalues are returned in descending order", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_true(all(diff(result$values) <= 0))
})

test_that("correct number of eigenvalues returned (kmax + 1)", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(length(result$values), 6)
})

test_that("correct number of eigenvectors returned (kmax + 1)", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(ncol(result$vectors), 6)
})

test_that("eigenvalues are positive", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_true(all(result$values > 0))
})

test_that("orientation is N when N <= T", {
  # T = 20, N = 10, so N <= T
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(result$orientation, "N")
})

test_that("orientation is T when N > T", {
  # T = 10, N = 20, so N > T
  X <- matrix(rnorm(200), 10, 20)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(result$orientation, "T")
})

test_that("eigenvector matrix has correct number of rows when orientation is N", {
  # N x N decomposition so vectors should have N rows = ncol(X)
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(nrow(result$vectors), ncol(X))  # N = ncol(X)
})

test_that("eigenvector matrix has correct number of rows when orientation is T", {
  # T x T decomposition so vectors should have T rows = nrow(X)
  X <- matrix(rnorm(200), 10, 20)
  X <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 5)
  expect_equal(nrow(result$vectors), nrow(X))  # T = nrow(X)
})

test_that("kmax too large generates warning and reduces kmax", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  expect_warning(
    .extract_eigenvalues(X, kmax = 50),
    "kmax"
  )
})

test_that("reduced kmax still returns correct structure", {
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)
  result <- suppressWarnings(.extract_eigenvalues(X, kmax = 50))
  expect_true(is.list(result))
  expect_named(result, c("values", "vectors", "orientation"))
})

test_that("eigenvalues match base R eigen on small matrix", {
  set.seed(42)
  X <- matrix(rnorm(200), 20, 10)
  X <- .prepare_matrix(X)

  # Our function
  result <- .extract_eigenvalues(X, kmax = 5)

  # Base R directly
  S    <- tcrossprod(X) / nrow(X)
  expected <- eigen(S, symmetric = TRUE)$values[1:6]

  expect_equal(result$values, expected, tolerance = 1e-10)
})

test_that("eigenvalues recover true factor structure", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3

  # Simulate factor model with 3 strong factors
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E

  X      <- .prepare_matrix(X)
  result <- .extract_eigenvalues(X, kmax = 8)

  # First 3 eigenvalues should be much larger than the rest
  gap <- result$values[k_true] / result$values[k_true + 1]
  expect_gt(gap, 5)
})

# tests/testthat/test-prepare_matrix.R

test_that(".prepare_matrix returns a matrix", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X)
  expect_true(is.matrix(result))
})

test_that(".prepare_matrix returns correct dimensions", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X)
  expect_equal(dim(result), c(20, 10))
})

test_that("individual demeaning removes column means", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X, demean = "individual", standardize = FALSE)
  col_means <- colMeans(result)
  expect_true(all(abs(col_means) < 1e-10))
})

test_that("time demeaning removes row means", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X, demean = "time", standardize = FALSE)
  row_means <- rowMeans(result)
  expect_true(all(abs(row_means) < 1e-10))
})

test_that("both demeaning removes column and row means", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X, demean = "both", standardize = FALSE)
  expect_true(all(abs(colMeans(result)) < 1e-10))
  expect_true(all(abs(rowMeans(result)) < 1e-10))
})

test_that("standardize scales columns to unit variance", {
  X <- matrix(rnorm(200), 20, 10)
  result <- .prepare_matrix(X, demean = "none", standardize = TRUE)
  col_sds <- apply(result, 2, sd)
  expect_true(all(abs(col_sds - 1) < 1e-10))
})

test_that("demean = none leaves means unchanged", {
  X <- matrix(rnorm(200), 20, 10)
  col_means_before <- colMeans(X)
  result <- .prepare_matrix(X, demean = "none", standardize = FALSE)
  expect_equal(colMeans(result), col_means_before)
})

test_that("non-matrix input is coerced to matrix", {
  X <- as.data.frame(matrix(rnorm(200), 20, 10))
  result <- .prepare_matrix(X)
  expect_true(is.matrix(result))
})

test_that("non-numeric input throws error", {
  X <- matrix(letters[1:20], 4, 5)
  expect_error(.prepare_matrix(X), "numeric")
})

test_that("missing values throw error", {
  X <- matrix(rnorm(200), 20, 10)
  X[1, 1] <- NA
  expect_error(.prepare_matrix(X), "missing")
})

test_that("single row matrix throws error", {
  X <- matrix(rnorm(10), 1, 10)
  expect_error(.prepare_matrix(X))
})

test_that("single column matrix throws error", {
  X <- matrix(rnorm(10), 10, 1)
  expect_error(.prepare_matrix(X))
})

test_that("zero variance column generates warning", {
  X <- matrix(rnorm(200), 20, 10)
  X[, 1] <- 5  # constant column
  expect_warning(
    .prepare_matrix(X, demean = "none", standardize = TRUE),
    "zero variance"
  )
})

#' Extract Leading Eigenvalues from a Panel Data Matrix
#'
#' Computes the leading eigenvalues of the sample covariance matrix using
#' a truncated eigendecomposition. Automatically selects the smaller of the
#' N x N or T x T covariance matrix for efficiency. Uses RSpectra when
#' available for large matrices, falling back to base R otherwise.
#'
#' @param X Numeric matrix of dimensions T x N.
#' @param kmax Integer. Number of leading eigenvalues to compute. Should be
#'   set generously (e.g., 8-15) to allow estimators to evaluate the full
#'   candidate range.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{values}{Numeric vector of length \code{kmax + 1} containing the
#'       leading eigenvalues in descending order. The extra eigenvalue is
#'       required by ratio-based estimators.}
#'     \item{vectors}{Numeric matrix of corresponding eigenvectors.}
#'     \item{orientation}{Character string, either \code{"N"} or \code{"T"},
#'       indicating which covariance matrix was decomposed.}
#'   }
#'
#' @details
#' When N <= T, the function decomposes the N x N matrix \eqn{X'X / T}.
#' When N > T, it decomposes the T x T matrix \eqn{XX' / N}.
#' This ensures the cheaper decomposition is always used.
#'
#' RSpectra's \code{eigs_sym()} is used when available and when
#' \code{min(N, T) > 100}, as truncated decomposition only provides
#' meaningful speedup at larger scales.
#'
#' @references
#' Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
#' Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
.extract_eigenvalues <- function(X, kmax) {

  T_dim <- nrow(X)
  N_dim <- ncol(X)

  # Validate kmax
  max_possible <- min(N_dim, T_dim) - 1
  if (kmax >= min(N_dim, T_dim)) {
    warning(
      "kmax (", kmax, ") is too large. ",
      "Reducing to min(N, T) - 1 = ", max_possible, "."
    )
    kmax <- max_possible
  }

  # Always decompose the smaller covariance matrix
  if (N_dim <= T_dim) {
    S           <- crossprod(X) / T_dim    # N x N  — X'X
    orientation <- "N"
  } else {
    S           <- tcrossprod(X) / N_dim   # T x T  — XX'
    orientation <- "T"
  }

  # Need kmax + 1 eigenvalues for ratio-based estimators
  k_compute <- kmax + 1

  # Use RSpectra if available and matrix is large enough to benefit
  use_rspectra <- requireNamespace("RSpectra", quietly = TRUE) &&
    nrow(S) > 100

  if (use_rspectra) {
    eig <- RSpectra::eigs_sym(S, k = k_compute)
  } else {
    eig_full    <- eigen(S, symmetric = TRUE)
    eig         <- list(
      values  = eig_full$values[1:k_compute],
      vectors = eig_full$vectors[, 1:k_compute, drop = FALSE]
    )
  }

  list(
    values      = eig$values,
    vectors     = eig$vectors,
    orientation = orientation
  )
}

#' Bai and Ng (2002) Information Criteria for Number of Factors
#'
#' Estimates the number of factors using the six penalty-based criteria
#' of Bai and Ng (2002). Includes three PC criteria (minimize penalized
#' residual variance) and three IC criteria (minimize penalized log residual
#' variance).
#'
#' @param eigenvalues Numeric vector of eigenvalues in descending order of
#'   length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.
#' @param kmax Integer. Maximum number of factors to consider.
#' @param N Integer. Number of cross-sectional units.
#' @param TT Integer. Number of time periods.
#' @param V0 Numeric scalar. Total mean squared value of the panel,
#'   \code{sum(X^2) / (N * TT)}, computed from unstandardized demeaned data.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k_pc1}{Integer. Selected number of factors by PC_p1.}
#'     \item{k_pc2}{Integer. Selected number of factors by PC_p2.}
#'     \item{k_pc3}{Integer. Selected number of factors by PC_p3.}
#'     \item{k_ic1}{Integer. Selected number of factors by IC_p1.}
#'     \item{k_ic2}{Integer. Selected number of factors by IC_p2.}
#'     \item{k_ic3}{Integer. Selected number of factors by IC_p3.}
#'     \item{pc1}{Numeric vector of length kmax. Full PC_p1 criterion sequence.}
#'     \item{pc2}{Numeric vector of length kmax. Full PC_p2 criterion sequence.}
#'     \item{pc3}{Numeric vector of length kmax. Full PC_p3 criterion sequence.}
#'     \item{ic1}{Numeric vector of length kmax. Full IC_p1 criterion sequence.}
#'     \item{ic2}{Numeric vector of length kmax. Full IC_p2 criterion sequence.}
#'     \item{ic3}{Numeric vector of length kmax. Full IC_p3 criterion sequence.}
#'   }
#'
#' @details
#'   The six criteria are defined as follows. Let \eqn{V(k)} denote the
#'   residual variance from a k-factor model, \eqn{m = \min(N, T)}, and
#'   \eqn{\hat{\sigma}^2 = V(k_{max})}.
#'
#'   PC criteria (minimize penalized residual variance):
#'   \deqn{PC_{p1}(k) = V(k) + k\hat{\sigma}^2 \frac{N+T}{NT} \ln\left(\frac{NT}{N+T}\right)}
#'   \deqn{PC_{p2}(k) = V(k) + k\hat{\sigma}^2 \frac{N+T}{NT} \ln(m)}
#'   \deqn{PC_{p3}(k) = V(k) + k\hat{\sigma}^2 \frac{\ln(m)}{m}}
#'
#'   IC criteria (minimize penalized log residual variance):
#'   \deqn{IC_{p1}(k) = \ln(V(k)) + k \frac{N+T}{NT} \ln\left(\frac{NT}{N+T}\right)}
#'   \deqn{IC_{p2}(k) = \ln(V(k)) + k \frac{N+T}{NT} \ln(m)}
#'   \deqn{IC_{p3}(k) = \ln(V(k)) + k \frac{\ln(m)}{m}}
#'
#'   \eqn{V(k)} is computed from the eigenvalues of \eqn{XX'/(NT)} as:
#'   \deqn{V(k) = \frac{1}{NT} \sum_{j=k+1}^{m} \lambda_j}
#'   which is the mean residual variance after removing the first k factors.
#'
#'   All six criteria are minimized over \eqn{k = 0, 1, \ldots, k_{max}}.
#'   Note that \eqn{k = 0} is included to allow for the possibility of no
#'   factors.
#'
#'   These estimators require both N and T to be large for consistent
#'   estimation. They may perform poorly when either dimension is small.
#'   For more robust estimation, consider \code{.ahn_horenstein()}.
#'
#' @references
#'   Bai, J. and Ng, S. (2002). Determining the Number of Factors in
#'   Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
#'
#' @seealso \code{.ahn_horenstein()}, \code{.extract_eigenvalues()},
#'   \code{\link{select_factors}}
.bai_ng <- function(eigenvalues, V0, kmax, N, TT) {

  m    <- min(N, TT)
  NT   <- N * TT

  # V(k) = V(0) - (1/N) * sum of top k eigenvalues of XX'/T
  cumulative <- cumsum(eigenvalues[1:kmax])
  V          <- V0 - c(0, cumulative) / N

  # sigma^2 hat = V(kmax)
  sigma2 <- V[kmax + 1]

  # Penalty terms
  g1 <- ((N + TT) / NT) * log(NT / (N + TT))
  g2 <- ((N + TT) / NT) * log(m)
  g3 <- log(m) / m

  ks <- 0:kmax

  # PC criteria
  pc1 <- V + ks * sigma2 * g1
  pc2 <- V + ks * sigma2 * g2
  pc3 <- V + ks * sigma2 * g3

  # IC criteria
  ic1 <- log(V) + ks * g1
  ic2 <- log(V) + ks * g2
  ic3 <- log(V) + ks * g3

  list(
    k_pc1 = which.min(pc1) - 1L,
    k_pc2 = which.min(pc2) - 1L,
    k_pc3 = which.min(pc3) - 1L,
    k_ic1 = which.min(ic1) - 1L,
    k_ic2 = which.min(ic2) - 1L,
    k_ic3 = which.min(ic3) - 1L,
    pc1   = pc1,
    pc2   = pc2,
    pc3   = pc3,
    ic1   = ic1,
    ic2   = ic2,
    ic3   = ic3
  )
}

#' Select the Number of Factors in an Approximate Factor Model
#'
#' A unified interface for estimating the number of factors in a large
#' dimensional approximate factor model. Preprocesses the data and dispatches
#' to one or more factor number estimators.
#'
#' @param X A numeric matrix of dimensions T x N (time periods x units), or
#'   an object coercible to a numeric matrix. Must be a balanced panel with no
#'   missing values.
#' @param method Character vector specifying which estimator(s) to use. One or
#'   more of \code{"ahn_horenstein"}, \code{"bai_ng"}, \code{"onatski_2009"},
#'   \code{"onatski_2010"}, \code{"abc"}, \code{"lam_yao"}. Defaults to
#'   \code{"ahn_horenstein"}.
#' @param kmax Integer. Maximum number of factors to consider. Defaults to
#'   \code{NULL}, in which case it is set to \code{min(floor(sqrt(min(N, T))),
#'   8)}.
#' @param demean Character string passed to \code{.prepare_matrix()}. One of
#'   \code{"both"}, \code{"individual"}, \code{"time"}, \code{"none"}.
#'   Defaults to \code{"both"} as recommended by Ahn and Horenstein (2013).
#' @param standardize Logical. Whether to standardize columns to unit variance
#'   before estimation. Defaults to \code{TRUE}. Note that \code{bai_ng},
#'   \code{abc}, and \code{lam_yao} always use unstandardized data regardless
#'   of this setting.
#' @param h Integer. Number of lags to use for the \code{lam_yao} estimator.
#'   Defaults to \code{1}. Ignored for all other methods.
#' @param alpha Numeric. Significance level for the \code{onatski_2009}
#'   sequential test. Defaults to \code{0.05}. Must be one of
#'   \code{0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.15}.
#'   Ignored for all other methods.
#'
#' @return An object of class \code{"factor_select"}, which is a named list
#'   with the following elements:
#'   \describe{
#'     \item{k}{Named integer vector of selected factor numbers, one per
#'       method requested.}
#'     \item{method}{Character vector of methods used.}
#'     \item{kmax}{Integer. Maximum number of factors considered.}
#'     \item{eigenvalues}{Numeric vector of leading eigenvalues.}
#'     \item{details}{Named list of full output from each estimator.}
#'     \item{call}{The matched call.}
#'   }
#'
#' @details
#'   The data are first preprocessed via \code{.prepare_matrix()} and then
#'   a single eigendecomposition is performed via \code{.extract_eigenvalues()},
#'   which is shared across all requested estimators for efficiency.
#'
#'   The default method is \code{"ahn_horenstein"}, which is recommended for
#'   most applications. It is robust to perturbations in the eigenvalue
#'   spectrum and performs well when only one of N or T is large.
#'
#'   The \code{"bai_ng"}, \code{"abc"}, and \code{"lam_yao"} methods always
#'   use unstandardized data because their penalty terms and auto-covariance
#'   structure depend on the actual scale of the data.
#'
#' @references
#'   Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
#'   Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
#'
#'   Bai, J. and Ng, S. (2002). Determining the Number of Factors in
#'   Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
#'
#'   Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
#'   for Determining the Number of Factors in Approximate Factor Models.
#'   \emph{Statistics and Probability Letters}, 80, 1806-1813.
#'
#'   Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional
#'   Time Series: Inference for the Number of Factors.
#'   \emph{The Annals of Statistics}, 40(2), 694-726.
#'
#' @seealso \code{.ahn_horenstein()}, \code{.bai_ng()},
#'   \code{.abc()}, \code{.lam_yao()},
#'   \code{.prepare_matrix()}, \code{.extract_eigenvalues()}
#'
#' @importFrom graphics abline legend
#' @export
#'
#' @examples
#' set.seed(42)
#' N <- 100; T <- 200; k_true <- 3
#' Lambda <- matrix(rnorm(N * k_true), N, k_true)
#' F_mat  <- matrix(rnorm(T * k_true), T, k_true)
#' E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
#' X      <- F_mat %*% t(Lambda) + E
#' select_factors(X)
select_factors <- function(X,
                           method      = "ahn_horenstein",
                           kmax        = NULL,
                           demean      = c("both", "individual", "time", "none"),
                           standardize = TRUE,
                           h           = 1L,
                           alpha       = 0.05) {

  call   <- match.call()
  demean <- match.arg(demean)

  # Validate method
  valid_methods <- c("ahn_horenstein", "bai_ng", "onatski_2009",
                     "onatski_2010", "abc", "lam_yao")
  bad <- setdiff(method, valid_methods)
  if (length(bad) > 0) {
    stop("Unknown method(s): ", paste(bad, collapse = ", "),
         "\nValid methods: ", paste(valid_methods, collapse = ", "))
  }

  # Validate h
  if (!is.numeric(h) || length(h) != 1 || h < 1) {
    stop("h must be a single positive integer.")
  }

  # Validate alpha
  valid_alphas <- c(0.15, 0.10, 0.09, 0.08, 0.07, 0.06,
                    0.05, 0.04, 0.03, 0.02, 0.01)
  if (!alpha %in% valid_alphas) {
    stop("alpha must be one of: ", paste(valid_alphas, collapse = ", "))
  }

  # Preprocess — standardized version for most estimators
  X_clean <- .prepare_matrix(X, demean = demean, standardize = standardize)

  T_dim <- nrow(X_clean)
  N_dim <- ncol(X_clean)
  n     <- min(N_dim, T_dim)

  # Set default kmax
  if (is.null(kmax)) {
    kmax <- min(floor(sqrt(n)), 8)
  }

  # Single eigendecomposition shared across standardized estimators
  eig <- .extract_eigenvalues(X_clean, kmax = kmax)

  # Bai & Ng, ABC and Lam-Yao require unstandardized data
  if (any(c("bai_ng", "abc", "lam_yao") %in% method)) {
    X_bn   <- .prepare_matrix(X, demean = demean, standardize = FALSE)
    V0     <- sum(X_bn^2) / (N_dim * T_dim)
    eig_bn <- .extract_eigenvalues(X_bn, kmax = kmax)
  }

  # Dispatch to each requested estimator
  details  <- list()
  k        <- integer(length(method))
  names(k) <- method

  for (m in method) {
    result <- switch(m,
                     ahn_horenstein = {
                       res  <- .ahn_horenstein(eig$values, kmax = kmax, n = n)
                       k[m] <- res$k_gr
                       res
                     },
                     bai_ng = {
                       res  <- .bai_ng(eig_bn$values, V0 = V0, kmax = kmax,
                                       N = N_dim, TT = T_dim)
                       k[m] <- res$k_ic1
                       res
                     },
                     abc = {
                       res  <- .abc(eig_bn$values, V0 = V0, kmax = kmax,
                                    N = N_dim, TT = T_dim)
                       k[m] <- res$k_abc1
                       res
                     },
                     lam_yao = {
                       res  <- .lam_yao(X_bn, kmax = kmax, h = h)
                       k[m] <- res$k
                       res
                     },
                     onatski_2009 = {
                       res  <- .onatski_2009(X_clean, kmax = kmax, alpha = alpha)
                       k[m] <- res$k
                       res
                     },
                     onatski_2010 = {
                       eig_ed <- .extract_eigenvalues(X_clean, kmax = kmax + 4)
                       res    <- .onatski_2010(eig_ed$values, kmax = kmax)
                       k[m]   <- res$k
                       res
                     }
    )
    details[[m]] <- result
  }

  structure(
    list(
      k           = k,
      method      = method,
      kmax        = kmax,
      eigenvalues = eig$values,
      details     = details,
      call        = call
    ),
    class = "factor_select"
  )
}


#' Print Method for factor_select Objects
#'
#' @param x A \code{factor_select} object.
#' @param ... Further arguments passed to or from other methods.
#' @return Invisibly returns \code{x}, called for its side effect of
#'   printing a summary of the factor selection results to the console.
#' @export
print.factor_select <- function(x, ...) {
  cat("Factor Number Selection\n")
  cat("=======================\n")
  cat("Call: "); print(x$call)
  cat("\nkmax:", x$kmax, "\n")
  cat("\nEstimated number of factors:\n")
  for (m in x$method) {
    cat(sprintf("  %-20s %d\n", m, x$k[m]))
  }
  invisible(x)
}


#' Summary Method for factor_select Objects
#'
#' @param object A \code{factor_select} object.
#' @param ... Further arguments passed to or from other methods.
#' @return Invisibly returns \code{object}, called for its side effect
#'   of printing a summary including leading eigenvalues to the console.
#' @export
summary.factor_select <- function(object, ...) {
  print(object)
  cat("\nLeading eigenvalues:\n")
  cat(round(object$eigenvalues, 4), "\n")
  invisible(object)
}


#' Plot Method for factor_select Objects
#'
#' Produces a scree plot of the leading eigenvalues with the selected
#' number of factors marked.
#'
#' @param x A \code{factor_select} object.
#' @param main Character string. Plot title. Defaults to \code{"Scree Plot"}.
#' @param ... Further arguments passed to \code{plot()}.
#' @return Invisibly returns \code{x}, called for its side effect of
#'   producing a scree plot.
#' @importFrom graphics abline legend
#' @export
plot.factor_select <- function(x, main = "Scree Plot", ...) {
  eigs <- x$eigenvalues
  k    <- x$k

  plot(seq_along(eigs), eigs,
       type = "b",
       xlab = "Factor Number",
       ylab = "Eigenvalue",
       main = main,
       pch  = 19, ...)

  cols <- c("red", "blue", "darkgreen", "purple", "orange", "brown")
  for (i in seq_along(x$method)) {
    abline(v   = k[x$method[i]],
           col = cols[i],
           lty = 2,
           lwd = 1.5)
  }

  legend("topright",
         legend = paste(x$method, "=", k),
         col    = cols[seq_along(x$method)],
         lty    = 2,
         lwd    = 1.5,
         bty    = "n")

  invisible(x)
}

# tests/testthat/test-select_factors.R

test_that("select_factors returns a factor_select object", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  result <- select_factors(X)
  expect_s3_class(result, "factor_select")
})

test_that("select_factors returns correct list elements", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  result <- select_factors(X)
  expect_named(result, c("k", "method", "kmax", "eigenvalues", "details", "call"))
})

test_that("default method is ahn_horenstein", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_equal(result$method, "ahn_horenstein")
})

test_that("default kmax is set automatically", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_true(!is.null(result$kmax))
  expect_true(result$kmax > 0)
})

test_that("user supplied kmax is respected", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X, kmax = 5)
  expect_equal(result$kmax, 5)
})

test_that("eigenvalues are returned", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X, kmax = 5)
  expect_true(is.numeric(result$eigenvalues))
  expect_true(length(result$eigenvalues) > 0)
})

test_that("k is a named integer vector", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_true(is.integer(result$k))
  expect_named(result$k, "ahn_horenstein")
})

test_that("k is within valid range", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X, kmax = 8)
  expect_true(all(result$k >= 1))
  expect_true(all(result$k <= 8))
})

test_that("unknown method throws informative error", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  expect_error(select_factors(X, method = "made_up"), "Unknown method")
})

test_that("details list contains entry for each method", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X, method = "ahn_horenstein")
  expect_true("ahn_horenstein" %in% names(result$details))
})

test_that("print method runs without error", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_output(print(result))
})

test_that("summary method runs without error", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_output(summary(result))
})

test_that("plot method runs without error", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  result <- select_factors(X)
  expect_silent(plot(result))
})

test_that("select_factors recovers true number of factors", {
  set.seed(42)
  N <- 100; T <- 200; k_true <- 3
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
  expect_equal(result$k[["ahn_horenstein"]], k_true)
})

test_that("select_factors works with small N large T", {
  set.seed(42)
  N <- 25; T <- 200; k_true <- 2
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
  expect_equal(result$k[["ahn_horenstein"]], k_true)
})

test_that("select_factors works with large N small T", {
  set.seed(42)
  N <- 200; T <- 25; k_true <- 2
  Lambda <- matrix(rnorm(N * k_true), N, k_true)
  F_mat  <- matrix(rnorm(T * k_true), T, k_true)
  E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
  X      <- F_mat %*% t(Lambda) + E
  result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
  expect_equal(result$k[["ahn_horenstein"]], k_true)
})

test_that("demean argument is passed through correctly", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  expect_no_error(select_factors(X, demean = "individual"))
  expect_no_error(select_factors(X, demean = "time"))
  expect_no_error(select_factors(X, demean = "none"))
  expect_no_error(select_factors(X, demean = "both"))
})

test_that("standardize = FALSE runs without error", {
  set.seed(42)
  X <- matrix(rnorm(2000), 200, 100)
  expect_no_error(select_factors(X, standardize = FALSE))
})

#' Lam and Yao (2012) Eigenvalue Ratio Estimator
#'
#' Estimates the number of factors using the eigenvalue ratio estimator of
#' Lam and Yao (2012). Unlike estimators based on the contemporaneous
#' covariance matrix, this estimator uses lagged auto-covariance matrices,
#' exploiting the fact that the factor loading space is spanned by the
#' eigenvectors of the summed lagged auto-covariance matrix M corresponding
#' to its nonzero eigenvalues.
#'
#' @param X Numeric matrix of dimensions T x N, typically preprocessed by
#'   \code{.prepare_matrix()}.
#' @param kmax Integer. Maximum number of factors to consider.
#' @param h Integer. Number of lags to use in constructing the auto-covariance
#'   matrix M. Defaults to \code{1}. The paper suggests small values are
#'   sufficient; increasing h may improve performance when factors have
#'   strong serial correlation.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k}{Integer. Selected number of factors.}
#'     \item{ratios}{Numeric vector of length kmax. Full eigenvalue ratio
#'       sequence of M.}
#'     \item{eigenvalues}{Numeric vector of length kmax + 1. Leading
#'       eigenvalues of M in descending order.}
#'   }
#'
#' @details
#'   The estimator constructs the N x N matrix:
#'   \deqn{M = \sum_{k=1}^{h} \hat{\Sigma}_k \hat{\Sigma}_k'}
#'   where \eqn{\hat{\Sigma}_k = T^{-1} \sum_{t=k+1}^{T} x_t x_{t-k}'} is
#'   the lag-k sample auto-covariance matrix.
#'
#'   The factor loading space is spanned by the eigenvectors of M
#'   corresponding to its nonzero eigenvalues, and the number of nonzero
#'   eigenvalues equals the number of factors r (Lam and Yao, 2012,
#'   Proposition 1). In finite samples, the ratio of adjacent eigenvalues
#'   of M spikes at r because eigenvalues r+1 onward are theoretically zero.
#'
#'   The number of factors is estimated as:
#'   \deqn{\hat{r} = \arg\max_{1 \leq k \leq k_{max}} \frac{\lambda_k(M)}{\lambda_{k+1}(M)}}
#'
#' @references
#'   Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional
#'   Time Series: Inference for the Number of Factors.
#'   \emph{The Annals of Statistics}, 40(2), 694-726.
#'
#' @seealso \code{.ahn_horenstein()}, \code{\link{select_factors}}
.lam_yao <- function(X, kmax, h = 1) {

  TT <- nrow(X)
  N  <- ncol(X)

  # Compute M = sum_{k=1}^{h} Sigma_k %*% t(Sigma_k)
  M <- matrix(0, N, N)

  for (lag in 1:h) {
    # Lag-k auto-covariance matrix: N x N
    # Sigma_k = (1/T) * t(X[(lag+1):T, ]) %*% X[1:(T-lag), ]
    X_lead <- X[(lag + 1):TT, ]   # T-lag x N
    X_lag  <- X[1:(TT - lag), ]   # T-lag x N
    Sigma_k <- crossprod(X_lead, X_lag) / TT   # N x N
    M <- M + Sigma_k %*% t(Sigma_k)
  }

  # Eigendecomposition of M — symmetric positive semidefinite
  eig <- eigen(M, symmetric = TRUE)

  # Leading kmax + 1 eigenvalues
  eigs <- eig$values[1:(kmax + 1)]

  # Direct ratio of adjacent eigenvalues
  ratios <- eigs[1:kmax] / eigs[2:(kmax + 1)]

  list(
    k           = which.max(ratios),
    ratios      = ratios,
    eigenvalues = eigs
  )
}

#' Ahn-Horenstein Eigenvalue Ratio Estimator
#'
#' Estimates the number of factors using the eigenvalue ratio (ER) and
#' growth ratio (GR) statistics of Ahn and Horenstein (2013). The ratio
#' approach provides robustness to perturbations in the eigenvalue spectrum
#' and performs well when only one dimension (N or T) is large.
#'
#' @param eigenvalues Numeric vector of eigenvalues in descending order of
#'   length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.
#'   Must be positive.
#' @param kmax Integer. Maximum number of factors to consider. The function
#'   evaluates the ratio statistics for k = 1, ..., kmax.
#' @param n Integer. The value of min(N, T), used to compute the mock
#'   eigenvalue boundary term following Ahn and Horenstein (2013) Corollary 1.
#'
#' @return A named list with the following elements:
#'   \describe{
#'     \item{k_er}{Integer. Selected number of factors based on the ER statistic.}
#'     \item{k_gr}{Integer. Selected number of factors based on the GR statistic.}
#'     \item{er}{Numeric vector of length kmax. Full ER statistic sequence.}
#'     \item{gr}{Numeric vector of length kmax. Full GR statistic sequence.}
#'   }
#'
#' @details
#'   The ER statistic is defined as the ratio of successive eigenvalue
#'   differences:
#'   \deqn{ER(k) = \delta_k / \delta_{k+1}}
#'   where \eqn{\delta_k} is the k-th successive difference in the eigenvalue
#'   sequence. The GR statistic replaces raw differences with log growth rates:
#'   \deqn{GR(k) = \log(1 + \delta_k / \lambda_k) / \log(1 + \delta_{k+1} / \lambda_{k+1})}
#'   The boundary case k = 0 is handled by assigning \eqn{\lambda_1 / \log(n)}
#'   as the initial difference term, following Ahn and Horenstein (2013).
#'
#'   The number of factors is selected as the argmax of each statistic over
#'   k = 1, ..., kmax.
#'
#' @references
#'   Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
#'   Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
#'
#' @seealso \code{.extract_eigenvalues()}, \code{\link{select_factors}}
.ahn_horenstein <- function(eigenvalues, kmax, n) {

  eigs <- eigenvalues[1:(kmax + 1)]

  # Mock eigenvalue for k=0: V(0)/log(m) where m = min(N,T)
  V0       <- sum(eigs)
  mu_tilde <- c(V0 / log(n), eigs)   # prepend mock eigenvalue

  # ER statistic: ratio of adjacent eigenvalues
  # ER(k) = mu(k) / mu(k+1), k = 1,...,kmax
  er <- mu_tilde[2:(kmax + 1)] / mu_tilde[3:(kmax + 2)]

  # GR statistic: ratio of log growth rates of V(k)
  # V(k) = sum of eigenvalues from k+1 to m
  # mu*_NTk = mu_NTk / V(k)
  V   <- rev(cumsum(rev(eigs)))         # V(k) for k = 0,...,kmax
  V   <- c(V0, V)                       # prepend V(0)
  mu_star <- mu_tilde[2:(kmax + 2)] / V[1:(kmax + 1)]

  gr <- log(1 + mu_star[1:kmax]) / log(1 + mu_star[2:(kmax + 1)])

  list(
    k_er = which.max(er),
    k_gr = which.max(gr),
    er   = er,
    gr   = gr
  )
}

---
title: "Introduction to factorselect"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to factorselect}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(factorselect)
```

## Introduction

The `factorselect` package implements six estimators for determining
the number of factors in large dimensional approximate factor models.
The estimators differ in their theoretical assumptions, computational
approach, and finite sample performance.

The recommended estimator for most applications is the Ahn and
Horenstein (2013) eigenvalue ratio estimator, which is robust to
perturbations in the eigenvalue spectrum and performs well when only
one of N or T is large.

## Simulating Factor Model Data

The package includes a helper function for simulating data from a
static approximate factor model:

$$X = F \Lambda' + E$$

where $F$ is a $T \times k$ matrix of factors, $\Lambda$ is an
$N \times k$ matrix of loadings, and $E$ is an $N \times T$ matrix
of idiosyncratic errors.

```{r simulate}
set.seed(42)
X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5)
dim(X)
```

## The Recommended Estimator: Ahn & Horenstein (2013)

The eigenvalue ratio (ER) and growth ratio (GR) estimators of Ahn and
Horenstein (2013) are obtained by maximizing the ratio of adjacent
eigenvalues of the sample covariance matrix. The ratio approach
provides robustness to perturbations in the eigenvalue spectrum.

A key advantage over Bai and Ng (2002) is that the Ahn-Horenstein
estimator works well when only one of N or T is large, not requiring
both dimensions to grow simultaneously.

```{r ahn_horenstein}
result <- select_factors(X, method = "ahn_horenstein", kmax = 8)
print(result)
```

## Comparing All Estimators

All six estimators can be run simultaneously by passing a vector of
method names:

```{r all_methods}
result_all <- select_factors(
  X,
  method = c("ahn_horenstein", "bai_ng", "abc",
             "lam_yao", "onatski_2009", "onatski_2010"),
  kmax   = 8
)
print(result_all)
```

## Scree Plot

The `plot` method produces a scree plot of the leading eigenvalues
with the selected number of factors marked for each estimator:

```{r scree, fig.width = 6, fig.height = 4}
result_ah <- select_factors(X, method = "ahn_horenstein", kmax = 8)
plot(result_ah, main = "Scree Plot — Ahn & Horenstein (2013)")
```

## Finite Sample Performance

To illustrate the finite sample performance of the estimators, we
run a small simulation study with 100 replications across three
sample size configurations.

```{r simulation, cache = TRUE}
set.seed(123)
n_reps  <- 100
k_true  <- 3
configs <- list(
  large_both  = list(N = 100, TT = 200),
  small_N     = list(N = 25,  TT = 200),
  small_T     = list(N = 200, TT = 25)
)

results <- lapply(configs, function(cfg) {
  estimates <- replicate(n_reps, {
    X <- simulate_factor_model(N = cfg$N, TT = cfg$TT,
                               k = k_true, sd = 0.5)
    res <- select_factors(X,
                          method = c("ahn_horenstein", "bai_ng",
                                     "onatski_2010"),
                          kmax   = 8)
    res$k
  })
  rowMeans(estimates == k_true)
})

# Percentage correct for each configuration
do.call(rbind, lapply(names(results), function(nm) {
  data.frame(
    config         = nm,
    ahn_horenstein = round(results[[nm]]["ahn_horenstein"] * 100),
    bai_ng         = round(results[[nm]]["bai_ng"] * 100),
    onatski_2010   = round(results[[nm]]["onatski_2010"] * 100)
  )
}))
```

The simulation confirms that Ahn and Horenstein (2013) performs well
across all three configurations, including when only one dimension is
large. Bai and Ng (2002) tends to be less reliable in the asymmetric
sample size cases.

## Notes on Individual Estimators

### Bai & Ng (2002) and ABC (2010)

These estimators use unstandardized data internally. The `select_factors`
function handles this automatically — users do not need to preprocess
data differently when requesting these methods.

### Lam & Yao (2012)

This estimator uses lagged auto-covariance matrices rather than the
contemporaneous covariance matrix. The number of lags `h` defaults to
1 but can be adjusted:

```{r lam_yao}
result_ly <- select_factors(X, method = "lam_yao", kmax = 8, h = 1)
print(result_ly)
```

### Onatski (2009)

This estimator performs a sequential hypothesis test. The significance
level `alpha` defaults to 0.05 but can be adjusted:

```{r onatski_2009}
result_o09 <- select_factors(X, method = "onatski_2009",
                              kmax = 8, alpha = 0.05)
print(result_o09)
```

### Onatski (2010)

The edge distribution estimator uses an iterative calibration
procedure to estimate the threshold separating systematic from
idiosyncratic eigenvalues:

```{r onatski_2010}
result_o10 <- select_factors(X, method = "onatski_2010", kmax = 8)
print(result_o10)
```

## References

Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. *Econometrica*, 81(3), 1203-1227.

Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. *Econometrica*, 70(1), 191-221.

Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
for Determining the Number of Factors in Approximate Factor Models.
*Statistics and Probability Letters*, 80, 1806-1813.

Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional Time
Series: Inference for the Number of Factors. *The Annals of Statistics*,
40(2), 694-726.

Onatski, A. (2009). Testing Hypotheses About the Number of Factors in
Large Factor Models. *Econometrica*, 77(5), 1447-1479.

Onatski, A. (2010). Determining the Number of Factors From Empirical
Distribution of Eigenvalues. *The Review of Economics and Statistics*,
92(4), 1004-1016.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ahn_horenstein.R
\name{.ahn_horenstein}
\alias{.ahn_horenstein}
\title{Ahn-Horenstein Eigenvalue Ratio Estimator}
\usage{
.ahn_horenstein(eigenvalues, kmax, n)
}
\arguments{
\item{eigenvalues}{Numeric vector of eigenvalues in descending order of
length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.
Must be positive.}

\item{kmax}{Integer. Maximum number of factors to consider. The function
evaluates the ratio statistics for k = 1, ..., kmax.}

\item{n}{Integer. The value of min(N, T), used to compute the mock
eigenvalue boundary term following Ahn and Horenstein (2013) Corollary 1.}
}
\value{
A named list with the following elements:
\describe{
\item{k_er}{Integer. Selected number of factors based on the ER statistic.}
\item{k_gr}{Integer. Selected number of factors based on the GR statistic.}
\item{er}{Numeric vector of length kmax. Full ER statistic sequence.}
\item{gr}{Numeric vector of length kmax. Full GR statistic sequence.}
}
}
\description{
Estimates the number of factors using the eigenvalue ratio (ER) and
growth ratio (GR) statistics of Ahn and Horenstein (2013). The ratio
approach provides robustness to perturbations in the eigenvalue spectrum
and performs well when only one dimension (N or T) is large.
}
\details{
The ER statistic is defined as the ratio of successive eigenvalue
differences:
\deqn{ER(k) = \delta_k / \delta_{k+1}}
where \eqn{\delta_k} is the k-th successive difference in the eigenvalue
sequence. The GR statistic replaces raw differences with log growth rates:
\deqn{GR(k) = \log(1 + \delta_k / \lambda_k) / \log(1 + \delta_{k+1} / \lambda_{k+1})}
The boundary case k = 0 is handled by assigning \eqn{\lambda_1 / \log(n)}
as the initial difference term, following Ahn and Horenstein (2013).

The number of factors is selected as the argmax of each statistic over
k = 1, ..., kmax.
}
\references{
Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
}
\seealso{
\code{.extract_eigenvalues()}, \code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abc.R
\name{.abc}
\alias{.abc}
\title{Alessi, Barigozzi and Capasso (2010) Tuned Information Criteria}
\usage{
.abc(eigenvalues, V0, kmax, N, TT, c_grid = seq(0, 1, by = 0.01))
}
\arguments{
\item{eigenvalues}{Numeric vector of eigenvalues in descending order of
length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.}

\item{V0}{Numeric scalar. Total mean squared value of the panel,
\code{sum(X^2) / (N * TT)}, computed from unstandardized demeaned data.}

\item{kmax}{Integer. Maximum number of factors to consider.}

\item{N}{Integer. Number of cross-sectional units.}

\item{TT}{Integer. Number of time periods.}

\item{c_grid}{Numeric vector. Grid of tuning constants over which to
evaluate stability. Defaults to \code{seq(0, 1, by = 0.01)}.}
}
\value{
A named list with the following elements:
\describe{
\item{k_abc1}{Integer. Selected number of factors using ABC with IC1 penalty.}
\item{k_abc2}{Integer. Selected number of factors using ABC with IC2 penalty.}
\item{k_abc3}{Integer. Selected number of factors using ABC with IC3 penalty.}
\item{k_grid_abc1}{Integer vector of length \code{length(c_grid)}.
Selected k for each value of c using IC1 penalty.}
\item{k_grid_abc2}{Integer vector of length \code{length(c_grid)}.
Selected k for each value of c using IC2 penalty.}
\item{k_grid_abc3}{Integer vector of length \code{length(c_grid)}.
Selected k for each value of c using IC3 penalty.}
\item{c_grid}{Numeric vector. The tuning constant grid used.}
}
}
\description{
Estimates the number of factors using the tuning-stability procedure of
Alessi, Barigozzi and Capasso (2010) applied to the three IC penalty
functions of Bai and Ng (2002). For each penalty function, a grid of
tuning constants is used and the most stable estimate across the grid
is selected as the final estimate.
}
\details{
The ABC estimator applies the tuning-stability procedure of Hallin and
Liska (2007) to the IC criteria of Bai and Ng (2002). For each tuning
constant \eqn{c} in the grid, a modified criterion is minimized:
\deqn{IC_j(k, c) = \ln(V(k)) + k \cdot c \cdot g_j(N, T)}
where \eqn{g_j} is the penalty function from \eqn{IC_{pj}} of Bai and
Ng (2002), for j = 1, 2, 3. The final estimate is the modal value of
\eqn{\hat{k}(c)} across the grid — the value of k that is selected
most frequently as c varies.

As with \code{.bai_ng()}, this estimator requires unstandardized
data. The argument \code{V0} should be computed from demeaned but
unstandardized data.

The ABC estimator generally outperforms the raw Bai & Ng IC criteria
in finite samples, particularly when errors are cross-sectionally
correlated.
}
\references{
Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
for Determining the Number of Factors in Approximate Factor Models.
\emph{Statistics and Probability Letters}, 80, 1806-1813.

Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.

Hallin, M. and Liska, R. (2007). Determining the Number of Factors in
the Generalized Dynamic Factor Model. \emph{Journal of the American
Statistical Association}, 102, 603-617.
}
\seealso{
\code{.bai_ng()}, \code{.extract_eigenvalues()},
\code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_eigenvalues.R
\name{.extract_eigenvalues}
\alias{.extract_eigenvalues}
\title{Extract Leading Eigenvalues from a Panel Data Matrix}
\usage{
.extract_eigenvalues(X, kmax)
}
\arguments{
\item{X}{Numeric matrix of dimensions T x N.}

\item{kmax}{Integer. Number of leading eigenvalues to compute. Should be
set generously (e.g., 8-15) to allow estimators to evaluate the full
candidate range.}
}
\value{
A named list with the following elements:
\describe{
\item{values}{Numeric vector of length \code{kmax + 1} containing the
leading eigenvalues in descending order. The extra eigenvalue is
required by ratio-based estimators.}
\item{vectors}{Numeric matrix of corresponding eigenvectors.}
\item{orientation}{Character string, either \code{"N"} or \code{"T"},
indicating which covariance matrix was decomposed.}
}
}
\description{
Computes the leading eigenvalues of the sample covariance matrix using
a truncated eigendecomposition. Automatically selects the smaller of the
N x N or T x T covariance matrix for efficiency. Uses RSpectra when
available for large matrices, falling back to base R otherwise.
}
\details{
When N <= T, the function decomposes the N x N matrix \eqn{X'X / T}.
When N > T, it decomposes the T x T matrix \eqn{XX' / N}.
This ensures the cheaper decomposition is always used.

RSpectra's \code{eigs_sym()} is used when available and when
\code{min(N, T) > 100}, as truncated decomposition only provides
meaningful speedup at larger scales.
}
\references{
Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bai_ng.R
\name{.bai_ng}
\alias{.bai_ng}
\title{Bai and Ng (2002) Information Criteria for Number of Factors}
\usage{
.bai_ng(eigenvalues, V0, kmax, N, TT)
}
\arguments{
\item{eigenvalues}{Numeric vector of eigenvalues in descending order of
length kmax + 1, typically obtained from \code{.extract_eigenvalues()}.}

\item{V0}{Numeric scalar. Total mean squared value of the panel,
\code{sum(X^2) / (N * TT)}, computed from unstandardized demeaned data.}

\item{kmax}{Integer. Maximum number of factors to consider.}

\item{N}{Integer. Number of cross-sectional units.}

\item{TT}{Integer. Number of time periods.}
}
\value{
A named list with the following elements:
\describe{
\item{k_pc1}{Integer. Selected number of factors by PC_p1.}
\item{k_pc2}{Integer. Selected number of factors by PC_p2.}
\item{k_pc3}{Integer. Selected number of factors by PC_p3.}
\item{k_ic1}{Integer. Selected number of factors by IC_p1.}
\item{k_ic2}{Integer. Selected number of factors by IC_p2.}
\item{k_ic3}{Integer. Selected number of factors by IC_p3.}
\item{pc1}{Numeric vector of length kmax. Full PC_p1 criterion sequence.}
\item{pc2}{Numeric vector of length kmax. Full PC_p2 criterion sequence.}
\item{pc3}{Numeric vector of length kmax. Full PC_p3 criterion sequence.}
\item{ic1}{Numeric vector of length kmax. Full IC_p1 criterion sequence.}
\item{ic2}{Numeric vector of length kmax. Full IC_p2 criterion sequence.}
\item{ic3}{Numeric vector of length kmax. Full IC_p3 criterion sequence.}
}
}
\description{
Estimates the number of factors using the six penalty-based criteria
of Bai and Ng (2002). Includes three PC criteria (minimize penalized
residual variance) and three IC criteria (minimize penalized log residual
variance).
}
\details{
The six criteria are defined as follows. Let \eqn{V(k)} denote the
residual variance from a k-factor model, \eqn{m = \min(N, T)}, and
\eqn{\hat{\sigma}^2 = V(k_{max})}.

PC criteria (minimize penalized residual variance):
\deqn{PC_{p1}(k) = V(k) + k\hat{\sigma}^2 \frac{N+T}{NT} \ln\left(\frac{NT}{N+T}\right)}
\deqn{PC_{p2}(k) = V(k) + k\hat{\sigma}^2 \frac{N+T}{NT} \ln(m)}
\deqn{PC_{p3}(k) = V(k) + k\hat{\sigma}^2 \frac{\ln(m)}{m}}

IC criteria (minimize penalized log residual variance):
\deqn{IC_{p1}(k) = \ln(V(k)) + k \frac{N+T}{NT} \ln\left(\frac{NT}{N+T}\right)}
\deqn{IC_{p2}(k) = \ln(V(k)) + k \frac{N+T}{NT} \ln(m)}
\deqn{IC_{p3}(k) = \ln(V(k)) + k \frac{\ln(m)}{m}}

\eqn{V(k)} is computed from the eigenvalues of \eqn{XX'/(NT)} as:
\deqn{V(k) = \frac{1}{NT} \sum_{j=k+1}^{m} \lambda_j}
which is the mean residual variance after removing the first k factors.

All six criteria are minimized over \eqn{k = 0, 1, \ldots, k_{max}}.
Note that \eqn{k = 0} is included to allow for the possibility of no
factors.

These estimators require both N and T to be large for consistent
estimation. They may perform poorly when either dimension is small.
For more robust estimation, consider \code{.ahn_horenstein()}.
}
\references{
Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
}
\seealso{
\code{.ahn_horenstein()}, \code{.extract_eigenvalues()},
\code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_factors.R
\name{plot.factor_select}
\alias{plot.factor_select}
\title{Plot Method for factor_select Objects}
\usage{
\method{plot}{factor_select}(x, main = "Scree Plot", ...)
}
\arguments{
\item{x}{A \code{factor_select} object.}

\item{main}{Character string. Plot title. Defaults to \code{"Scree Plot"}.}

\item{...}{Further arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{x}, called for its side effect of
producing a scree plot.
}
\description{
Produces a scree plot of the leading eigenvalues with the selected
number of factors marked.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_factors.R
\name{print.factor_select}
\alias{print.factor_select}
\title{Print Method for factor_select Objects}
\usage{
\method{print}{factor_select}(x, ...)
}
\arguments{
\item{x}{A \code{factor_select} object.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x}, called for its side effect of
printing a summary of the factor selection results to the console.
}
\description{
Print Method for factor_select Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate_factor_model}
\alias{simulate_factor_model}
\title{Simulate Data from an Approximate Factor Model}
\usage{
simulate_factor_model(N, TT, k, sd = 1, seed = NULL)
}
\arguments{
\item{N}{Integer. Number of cross-sectional units.}

\item{TT}{Integer. Number of time periods. Named \code{TT} to avoid
conflict with the base R function \code{T} (which evaluates to
\code{TRUE}).}

\item{k}{Integer. True number of factors.}

\item{sd}{Numeric. Standard deviation of the idiosyncratic error term.
Defaults to \code{1}. Lower values produce stronger signal relative
to noise.}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.
Defaults to \code{NULL} (no seed set).}
}
\value{
A numeric matrix of dimensions \code{TT x N} generated from:
\deqn{X = F \Lambda' + E}
where \eqn{F} is a \code{TT x k} matrix of factors drawn from
\eqn{N(0,1)}, \eqn{\Lambda} is an \code{N x k} matrix of loadings
drawn from \eqn{N(0,1)}, and \eqn{E} is a \code{TT x N} matrix of
idiosyncratic errors drawn from \eqn{N(0, sd^2)}.
}
\description{
Generates a simulated panel data matrix from a static approximate factor
model. Useful for testing and benchmarking factor number estimators.
}
\details{
The data generating process follows the standard approximate factor
model of Chamberlain and Rothschild (1983) as used in the simulation
exercises of Ahn and Horenstein (2013). Factors and loadings are
independent standard normal draws. Errors are i.i.d. normal with
mean zero and standard deviation \code{sd}.

The signal-to-noise ratio is controlled by \code{sd} — smaller values
produce a cleaner factor structure that is easier for estimators to
recover. The default \code{sd = 1} matches the baseline simulation
design of Ahn and Horenstein (2013) with \code{theta = 1}.
}
\examples{
# Simulate a factor model with 3 factors
X <- simulate_factor_model(N = 100, TT = 200, k = 3, sd = 0.5, seed = 42)
dim(X)

# Pass directly to select_factors
result <- select_factors(X)
result$k
}
\references{
Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.

Chamberlain, G. and Rothschild, M. (1983). Arbitrage, Factor Structure,
and Mean-Variance Analysis on Large Asset Markets.
\emph{Econometrica}, 51(5), 1281-1304.
}
\seealso{
\code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onatski_2010.R
\name{.onatski_2010}
\alias{.onatski_2010}
\title{Onatski (2010) Edge Distribution Estimator}
\usage{
.onatski_2010(eigenvalues, kmax, n_iter = 4L)
}
\arguments{
\item{eigenvalues}{Numeric vector of eigenvalues in descending order,
typically obtained from \code{.extract_eigenvalues()}. Must
contain at least \code{kmax + 5} elements to allow the OLS
regression in the calibration step.}

\item{kmax}{Integer. Maximum number of factors to consider.}

\item{n_iter}{Integer. Maximum number of iterations for the
calibration procedure. Defaults to \code{4} as recommended
by Onatski (2010).}
}
\value{
A named list with the following elements:
\describe{
\item{k}{Integer. Estimated number of factors.}
\item{delta}{Numeric. The estimated threshold \eqn{\delta = 2|\hat{\beta}|}.}
\item{beta}{Numeric. The estimated slope coefficient \eqn{\hat{\beta}}
from the OLS regression in the final iteration.}
\item{differences}{Numeric vector of length kmax. Successive
eigenvalue differences \eqn{\lambda_i - \lambda_{i+1}}.}
\item{n_iter}{Integer. Number of iterations performed.}
}
}
\description{
Estimates the number of factors using the Edge Distribution (ED)
estimator of Onatski (2010). The estimator exploits the fact that
idiosyncratic eigenvalues of the sample covariance matrix cluster
around a single point, while systematic eigenvalues diverge to
infinity. The threshold separating the two groups is estimated
iteratively using the square root shape of the edge of the
eigenvalue distribution.
}
\details{
The ED estimator of Onatski (2010) is based on the theoretical result
that idiosyncratic eigenvalues cluster around the upper edge
\eqn{u(\mathcal{F}^{c,A,B})} of the limiting spectral distribution,
while systematic eigenvalues diverge. Near the edge, the density of
the limiting spectral distribution behaves like a square root function,
implying that eigenvalue differences \eqn{\lambda_i - \lambda_{i+1}}
for idiosyncratic eigenvalues behave approximately as
\eqn{(an)^{-2/3}}.

The calibration procedure estimates \eqn{\hat{\beta} = (an)^{-2/3}}
by regressing five consecutive eigenvalues \eqn{\lambda_j, \ldots,
  \lambda_{j+4}} on a constant and \eqn{(j-1)^{2/3}, \ldots,
  (j+3)^{2/3}}, where \eqn{j} is initialized at \eqn{r_{max} + 1}
and updated iteratively.

The estimator requires \code{eigenvalues} to contain at least
\code{kmax + 5} elements so that the OLS window \eqn{j, \ldots, j+4}
is always available.
}
\references{
Onatski, A. (2010). Determining the Number of Factors From Empirical
Distribution of Eigenvalues. \emph{The Review of Economics and
Statistics}, 92(4), 1004-1016.
}
\seealso{
\code{.extract_eigenvalues()}, \code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onatski_2009.R
\name{.onatski_2009}
\alias{.onatski_2009}
\title{Onatski (2009) Test for the Number of Factors}
\usage{
.onatski_2009(X, kmax, alpha = 0.05)
}
\arguments{
\item{X}{Numeric matrix of dimensions T x N, typically preprocessed by
\code{.prepare_matrix()}. Must have an even number of rows.}

\item{kmax}{Integer. Maximum number of factors to consider. Defines the
upper bound k1 in the sequential testing procedure.}

\item{alpha}{Numeric. Significance level for the sequential test.
Defaults to \code{0.05}. Must be one of \code{0.01, 0.02, 0.03, 0.04,
  0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.15}.}
}
\value{
A named list with the following elements:
\describe{
\item{k}{Integer. Estimated number of factors from the sequential
testing procedure.}
\item{ratios}{Numeric vector of length kmax. The ratio statistic
\eqn{(\tilde{\gamma}_i - \tilde{\gamma}_{i+1}) /
      (\tilde{\gamma}_{i+1} - \tilde{\gamma}_{i+2})} for each i.}
\item{eigenvalues}{Numeric vector of length kmax + 2. Leading
eigenvalues of the complex covariance matrix in descending order.}
\item{critical_value}{Numeric. Critical value used for the test
at the specified significance level.}
\item{alpha}{Numeric. The significance level used.}
}
}
\description{
Estimates the number of factors using the sequential hypothesis testing
procedure of Onatski (2009), applied to the static approximate factor
model version described in Section 4 of that paper. The test statistic
is based on ratios of differences of adjacent eigenvalues of a
complex-valued transformation of the data.
}
\details{
The static approximate factor model version of the Onatski (2009) test
(Section 4) proceeds as follows:

\enumerate{
\item Split the T x N data matrix into two halves of length T/2.
\item Form complex-valued vectors
\eqn{\tilde{X}_j = X_j + i X_{j + T/2}} for \eqn{j = 1, \ldots, T/2}.
\item Compute eigenvalues \eqn{\tilde{\gamma}_i} of
\eqn{\frac{2}{T} \sum_{j=1}^{T/2} \tilde{X}_j \tilde{X}_j^*}.
\item Sequentially test \eqn{H_0: r = k_0} versus
\eqn{H_1: k_0 < r \leq k_{max}} for \eqn{k_0 = 0, 1, \ldots}
using the statistic
\eqn{\tilde{R} = \max_{k_0 < i \leq k_{max}}
      (\tilde{\gamma}_i - \tilde{\gamma}_{i+1}) /
      (\tilde{\gamma}_{i+1} - \tilde{\gamma}_{i+2})}.
\item Stop when \eqn{H_0} is not rejected. The estimate is the
current \eqn{k_0}.
}

Critical values are taken from Table I of Onatski (2009) and depend
on the significance level \code{alpha} and the number of factors
tested under the alternative \eqn{k_1 - k_0 = k_{max} - k_0}.

If T is odd, the last observation is dropped to ensure equal-length
halves.
}
\references{
Onatski, A. (2009). Testing Hypotheses About the Number of Factors
in Large Factor Models. \emph{Econometrica}, 77(5), 1447-1479.
}
\seealso{
\code{.ahn_horenstein()}, \code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_matrix.R
\name{.prepare_matrix}
\alias{.prepare_matrix}
\title{Demean and Scale a Matrix for Factor Analysis}
\usage{
.prepare_matrix(
  X,
  demean = c("both", "individual", "time", "none"),
  standardize = TRUE
)
}
\arguments{
\item{X}{Numeric matrix of dimensions T x N (time periods x units).}

\item{demean}{Character string specifying the demeaning method. One of:
\describe{
\item{"both"}{Remove both individual (column) and time (row) means.
This is the recommended default for macro panels.}
\item{"individual"}{Remove individual (column) means only.}
\item{"time"}{Remove time (row) means only.}
\item{"none"}{No demeaning applied.}
}}

\item{standardize}{Logical. If \code{TRUE} (default), scale each column
to unit variance after demeaning.}
}
\value{
A demeaned and optionally scaled numeric matrix of the same
dimensions as \code{X}.
}
\description{
Removes individual means, time means, or both from a numeric matrix,
and optionally scales to unit variance. This is the standard preprocessing
step required before eigendecomposition in factor number estimation.
}
\details{
When \code{demean = "both"}, the function iterates individual and time
demeaning to convergence. This follows the within-transformation used in
panel data models.
}
\references{
Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.
}
\seealso{
\code{\link{select_factors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_factors.R
\name{summary.factor_select}
\alias{summary.factor_select}
\title{Summary Method for factor_select Objects}
\usage{
\method{summary}{factor_select}(object, ...)
}
\arguments{
\item{object}{A \code{factor_select} object.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{object}, called for its side effect
of printing a summary including leading eigenvalues to the console.
}
\description{
Summary Method for factor_select Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_factors.R
\name{select_factors}
\alias{select_factors}
\title{Select the Number of Factors in an Approximate Factor Model}
\usage{
select_factors(
  X,
  method = "ahn_horenstein",
  kmax = NULL,
  demean = c("both", "individual", "time", "none"),
  standardize = TRUE,
  h = 1L,
  alpha = 0.05
)
}
\arguments{
\item{X}{A numeric matrix of dimensions T x N (time periods x units), or
an object coercible to a numeric matrix. Must be a balanced panel with no
missing values.}

\item{method}{Character vector specifying which estimator(s) to use. One or
more of \code{"ahn_horenstein"}, \code{"bai_ng"}, \code{"onatski_2009"},
\code{"onatski_2010"}, \code{"abc"}, \code{"lam_yao"}. Defaults to
\code{"ahn_horenstein"}.}

\item{kmax}{Integer. Maximum number of factors to consider. Defaults to
\code{NULL}, in which case it is set to \code{min(floor(sqrt(min(N, T))),
  8)}.}

\item{demean}{Character string passed to \code{.prepare_matrix()}. One of
\code{"both"}, \code{"individual"}, \code{"time"}, \code{"none"}.
Defaults to \code{"both"} as recommended by Ahn and Horenstein (2013).}

\item{standardize}{Logical. Whether to standardize columns to unit variance
before estimation. Defaults to \code{TRUE}. Note that \code{bai_ng},
\code{abc}, and \code{lam_yao} always use unstandardized data regardless
of this setting.}

\item{h}{Integer. Number of lags to use for the \code{lam_yao} estimator.
Defaults to \code{1}. Ignored for all other methods.}

\item{alpha}{Numeric. Significance level for the \code{onatski_2009}
sequential test. Defaults to \code{0.05}. Must be one of
\code{0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.15}.
Ignored for all other methods.}
}
\value{
An object of class \code{"factor_select"}, which is a named list
with the following elements:
\describe{
\item{k}{Named integer vector of selected factor numbers, one per
method requested.}
\item{method}{Character vector of methods used.}
\item{kmax}{Integer. Maximum number of factors considered.}
\item{eigenvalues}{Numeric vector of leading eigenvalues.}
\item{details}{Named list of full output from each estimator.}
\item{call}{The matched call.}
}
}
\description{
A unified interface for estimating the number of factors in a large
dimensional approximate factor model. Preprocesses the data and dispatches
to one or more factor number estimators.
}
\details{
The data are first preprocessed via \code{.prepare_matrix()} and then
a single eigendecomposition is performed via \code{.extract_eigenvalues()},
which is shared across all requested estimators for efficiency.

The default method is \code{"ahn_horenstein"}, which is recommended for
most applications. It is robust to perturbations in the eigenvalue
spectrum and performs well when only one of N or T is large.

The \code{"bai_ng"}, \code{"abc"}, and \code{"lam_yao"} methods always
use unstandardized data because their penalty terms and auto-covariance
structure depend on the actual scale of the data.
}
\examples{
set.seed(42)
N <- 100; T <- 200; k_true <- 3
Lambda <- matrix(rnorm(N * k_true), N, k_true)
F_mat  <- matrix(rnorm(T * k_true), T, k_true)
E      <- matrix(rnorm(T * N, sd = 0.5), T, N)
X      <- F_mat \%*\% t(Lambda) + E
select_factors(X)
}
\references{
Ahn, S.C. and Horenstein, A.R. (2013). Eigenvalue Ratio Test for the
Number of Factors. \emph{Econometrica}, 81(3), 1203-1227.

Bai, J. and Ng, S. (2002). Determining the Number of Factors in
Approximate Factor Models. \emph{Econometrica}, 70(1), 191-221.

Alessi, L., Barigozzi, M. and Capasso, M. (2010). Improved Penalization
for Determining the Number of Factors in Approximate Factor Models.
\emph{Statistics and Probability Letters}, 80, 1806-1813.

Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional
Time Series: Inference for the Number of Factors.
\emph{The Annals of Statistics}, 40(2), 694-726.
}
\seealso{
\code{.ahn_horenstein()}, \code{.bai_ng()},
\code{.abc()}, \code{.lam_yao()},
\code{.prepare_matrix()}, \code{.extract_eigenvalues()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lam_yao.R
\name{.lam_yao}
\alias{.lam_yao}
\title{Lam and Yao (2012) Eigenvalue Ratio Estimator}
\usage{
.lam_yao(X, kmax, h = 1)
}
\arguments{
\item{X}{Numeric matrix of dimensions T x N, typically preprocessed by
\code{.prepare_matrix()}.}

\item{kmax}{Integer. Maximum number of factors to consider.}

\item{h}{Integer. Number of lags to use in constructing the auto-covariance
matrix M. Defaults to \code{1}. The paper suggests small values are
sufficient; increasing h may improve performance when factors have
strong serial correlation.}
}
\value{
A named list with the following elements:
\describe{
\item{k}{Integer. Selected number of factors.}
\item{ratios}{Numeric vector of length kmax. Full eigenvalue ratio
sequence of M.}
\item{eigenvalues}{Numeric vector of length kmax + 1. Leading
eigenvalues of M in descending order.}
}
}
\description{
Estimates the number of factors using the eigenvalue ratio estimator of
Lam and Yao (2012). Unlike estimators based on the contemporaneous
covariance matrix, this estimator uses lagged auto-covariance matrices,
exploiting the fact that the factor loading space is spanned by the
eigenvectors of the summed lagged auto-covariance matrix M corresponding
to its nonzero eigenvalues.
}
\details{
The estimator constructs the N x N matrix:
\deqn{M = \sum_{k=1}^{h} \hat{\Sigma}_k \hat{\Sigma}_k'}
where \eqn{\hat{\Sigma}_k = T^{-1} \sum_{t=k+1}^{T} x_t x_{t-k}'} is
the lag-k sample auto-covariance matrix.

The factor loading space is spanned by the eigenvectors of M
corresponding to its nonzero eigenvalues, and the number of nonzero
eigenvalues equals the number of factors r (Lam and Yao, 2012,
Proposition 1). In finite samples, the ratio of adjacent eigenvalues
of M spikes at r because eigenvalues r+1 onward are theoretically zero.

The number of factors is estimated as:
\deqn{\hat{r} = \arg\max_{1 \leq k \leq k_{max}} \frac{\lambda_k(M)}{\lambda_{k+1}(M)}}
}
\references{
Lam, C. and Yao, Q. (2012). Factor Modelling for High-Dimensional
Time Series: Inference for the Number of Factors.
\emph{The Annals of Statistics}, 40(2), 694-726.
}
\seealso{
\code{.ahn_horenstein()}, \code{\link{select_factors}}
}
